Method: RuboCop::Cop::Base.support_multiple_source?

Defined in:
lib/rubocop/cop/base.rb

.support_multiple_source?Boolean

Override if your cop should be called repeatedly for multiple investigations Between calls to on_new_investigation and on_investigation_end, the result of processed_source will remain constant. You should invalidate any caches that depend on the current processed_source in the on_new_investigation callback. If your cop does autocorrections, be aware that your instance may be called multiple times with the same processed_source.path but different content.

Returns:

  • (Boolean)


129
130
131
# File 'lib/rubocop/cop/base.rb', line 129

def self.support_multiple_source?
  false
end