Method: RuboCop::MagicComment::VimComment#encoding

Defined in:
lib/rubocop/magic_comment.rb

#encodingObject

For some reason the fileencoding keyword only works if there is at least one other token included in the string. For example

# works
  # vim: foo=bar, fileencoding=ascii-8bit

# does nothing
  # vim: foo=bar, fileencoding=ascii-8bit


251
252
253
# File 'lib/rubocop/magic_comment.rb', line 251

def encoding
  super if tokens.size > 1
end