Class: RuboCop::Config
Overview
This class represents the configuration of the RuboCop application
and all its cops. A Config is associated with a YAML configuration
file from which it was read. Several different Configs can be used
during a run of the rubocop program, if files in several
directories are inspected.
Defined Under Namespace
Classes: CopConfig
Constant Summary
collapse
- EMPTY_CONFIG =
{}.freeze
- DEFAULT_RAILS_VERSION =
5.0
Constants included
from PathUtil
PathUtil::HIDDEN_FILE_PATTERN
Instance Attribute Summary collapse
Class Method Summary
collapse
Instance Method Summary
collapse
Methods included from FileFinder
#find_file_upwards, #find_last_file_upwards, #traverse_directories_upwards
Methods included from PathUtil
absolute?, glob?, hidden_dir?, hidden_file?, hidden_file_in_not_hidden_dir?, match_path?, maybe_hidden_file?, pwd, relative_path, remote_file?, reset_pwd, smart_path
Constructor Details
#initialize(hash = RuboCop::ConfigLoader.default_configuration, loaded_path = nil) ⇒ Config
rubocop:disable-next Metrics/AbcSize, Metrics/MethodLength
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
|
# File 'lib/rubocop/config.rb', line 31
def initialize(hash = RuboCop::ConfigLoader.default_configuration, loaded_path = nil)
@loaded_path = loaded_path
@for_cop = Hash.new do |h, cop|
cop_name = cop.respond_to?(:cop_name) ? cop.cop_name : cop
if ConfigObsoletion.deprecated_cop_name?(cop)
cop_options = self[cop_name].dup || {}
else
qualified_cop_name = Cop::Registry.qualified_cop_name(cop_name, loaded_path, warn: false)
cop_options = self[qualified_cop_name].dup || {}
cop_options['Enabled'] = enable_cop?(qualified_cop_name, cop_options)
ConfigObsoletion.deprecated_names_for(cop).each do |deprecated_cop_name|
deprecated_config = @for_cop[deprecated_cop_name]
next if deprecated_config.empty?
warn Rainbow(<<~WARNING).yellow
Warning: Using `#{deprecated_cop_name}` configuration in #{loaded_path} for `#{cop}`.
WARNING
cop_options.merge!(@for_cop[deprecated_cop_name])
end
end
h[cop] = h[cop_name] = cop_options
end
@hash = hash
@validator = ConfigValidator.new(self)
@badge_config_cache = {}.compare_by_identity
@clusivity_config_exists_cache = {}
end
|
Instance Attribute Details
#loaded_path ⇒ Object
Returns the value of attribute loaded_path.
21
22
23
|
# File 'lib/rubocop/config.rb', line 21
def loaded_path
@loaded_path
end
|
Class Method Details
.create(hash, path, check: true) ⇒ Object
23
24
25
26
27
28
|
# File 'lib/rubocop/config.rb', line 23
def self.create(hash, path, check: true)
config = new(hash, path)
config.check if check
config
end
|
Instance Method Details
#active_support_extensions_enabled? ⇒ Boolean
239
240
241
|
# File 'lib/rubocop/config.rb', line 239
def active_support_extensions_enabled?
for_all_cops['ActiveSupportExtensionsEnabled']
end
|
#add_excludes_from_higher_level(highest_config) ⇒ Object
126
127
128
129
130
131
132
133
134
135
136
|
# File 'lib/rubocop/config.rb', line 126
def add_excludes_from_higher_level(highest_config)
return unless highest_config.for_all_cops['Exclude']
excludes = for_all_cops['Exclude'] ||= []
highest_config.for_all_cops['Exclude'].each do |path|
unless path.is_a?(Regexp) || absolute?(path)
path = File.join(File.dirname(highest_config.loaded_path), path)
end
excludes << path unless excludes.include?(path)
end
end
|
#allowed_camel_case_file?(file) ⇒ Boolean
267
268
269
270
271
272
273
274
|
# File 'lib/rubocop/config.rb', line 267
def allowed_camel_case_file?(file)
return true if File.extname(file) == '.gemspec'
file_to_include?(file) { |pattern| /[A-Z]/.match?(pattern.to_s) }
end
|
#base_dir_for_path_parameters ⇒ Object
Paths specified in configuration files starting with .rubocop are
relative to the directory where that file is. Paths in other config files
are relative to the current directory. This is so that paths in
config/default.yml, for example, are not relative to RuboCop's config
directory since that wouldn't work.
308
309
310
311
312
313
314
315
316
|
# File 'lib/rubocop/config.rb', line 308
def base_dir_for_path_parameters
@base_dir_for_path_parameters ||=
if loaded_path && File.basename(loaded_path).start_with?('.rubocop') &&
loaded_path != File.join(Dir.home, ConfigLoader::DOTFILE)
File.expand_path(File.dirname(loaded_path))
else
PathUtil.pwd
end
end
|
#bundler_lock_file_path ⇒ String?
338
339
340
341
342
343
344
345
346
347
|
# File 'lib/rubocop/config.rb', line 338
def bundler_lock_file_path
return nil unless loaded_path
base_path = base_dir_for_path_parameters
['Gemfile.lock', 'gems.locked'].each do |file_name|
path = find_file_upwards(file_name, base_path)
return path if path
end
nil
end
|
#check ⇒ Object
84
85
86
87
88
89
|
# File 'lib/rubocop/config.rb', line 84
def check
deprecation_check { |deprecation_message| warn("#{loaded_path} - #{deprecation_message}") }
@validator.validate
make_excludes_absolute
self
end
|
#clusivity_config_for_badge?(badge) ⇒ Boolean
This method is part of a private API.
You should avoid using this method if possible, as it may be removed or be changed in the future.
Returns whether config for this badge has 'Include' or 'Exclude' keys.
183
184
185
186
187
188
189
|
# File 'lib/rubocop/config.rb', line 183
def clusivity_config_for_badge?(badge)
exists = @clusivity_config_exists_cache[badge.to_s]
return exists unless exists.nil?
cop_config = for_badge(badge)
@clusivity_config_exists_cache[badge.to_s] = cop_config['Include'] || cop_config['Exclude']
end
|
#cop_enabled?(name) ⇒ Boolean
203
204
205
|
# File 'lib/rubocop/config.rb', line 203
def cop_enabled?(name)
!!for_cop(name)['Enabled']
end
|
#deprecation_check ⇒ Object
138
139
140
141
142
143
144
145
146
147
|
# File 'lib/rubocop/config.rb', line 138
def deprecation_check
%w[Exclude Include].each do |key|
plural = "#{key}s"
next unless for_all_cops[plural]
for_all_cops[key] = for_all_cops[plural] for_all_cops.delete(plural)
yield "AllCops/#{plural} was renamed to AllCops/#{key}"
end
end
|
#disabled_new_cops? ⇒ Boolean
207
208
209
|
# File 'lib/rubocop/config.rb', line 207
def disabled_new_cops?
for_all_cops['NewCops'] == 'disable'
end
|
#enabled_new_cop?(qualified_cop_name) ⇒ Boolean
Whether the given pending cop should be enabled, based on the NewCops setting of
its department (if any) or of AllCops. A department may set NewCops to enable,
disable, pending, or a version, in which case pending cops added in that version
or earlier are enabled.
219
220
221
222
223
224
225
226
227
|
# File 'lib/rubocop/config.rb', line 219
def enabled_new_cop?(qualified_cop_name)
setting = new_cops_setting_for(qualified_cop_name)
case setting.to_s
when 'enable' then true
when '', 'pending', 'disable' then false
else new_cops_version_covers?(setting, qualified_cop_name)
end
end
|
#enabled_new_cops? ⇒ Boolean
211
212
213
|
# File 'lib/rubocop/config.rb', line 211
def enabled_new_cops?
for_all_cops['NewCops'] == 'enable'
end
|
#file_to_exclude?(file) ⇒ Boolean
286
287
288
289
|
# File 'lib/rubocop/config.rb', line 286
def file_to_exclude?(file)
file = File.expand_path(file)
patterns_to_exclude.any? { |pattern| match_path?(pattern, file) }
end
|
#file_to_include?(file) ⇒ Boolean
Returns true if the file matches any include pattern. If a block is given, the block is called
to determine if the pattern is relevant (true returned by the block) or should be skipped
(false returned).
250
251
252
253
254
255
256
257
258
259
260
261
262
263
264
265
|
# File 'lib/rubocop/config.rb', line 250
def file_to_include?(file)
relative_file_path = path_relative_to_config(file)
is_hidden = relative_file_path.start_with?('.') && !relative_file_path.start_with?('..')
return false if is_hidden && !possibly_include_hidden?
absolute_file_path = File.expand_path(file)
patterns_to_include.any? do |pattern|
next if block_given? && !yield(pattern)
match_relative_or_absolute_path?(pattern, relative_file_path, absolute_file_path)
end
end
|
#for_all_cops ⇒ Object
199
200
201
|
# File 'lib/rubocop/config.rb', line 199
def for_all_cops
@for_all_cops ||= self['AllCops'] || {}
end
|
#for_badge(badge) ⇒ Config
Note: the 'Enabled' attribute is same as that returned by for_cop
165
166
167
168
169
170
171
172
173
174
175
176
177
178
179
|
# File 'lib/rubocop/config.rb', line 165
def for_badge(badge)
@badge_config_cache[badge] ||= begin
department_config = self[badge.department_name]
cop_config = for_cop(badge.to_s)
if department_config
merged_config = department_config.merge(cop_config)
if department_config['Exclude'] && cop_config['Exclude']
merged_config['Exclude'] = department_config['Exclude'] | cop_config['Exclude']
end
merged_config
else
cop_config
end
end
end
|
#for_cop(cop) ⇒ Config
Note: the 'Enabled' attribute is calculated according to the department's
and 'AllCops' configuration; other attributes are not inherited.
152
153
154
|
# File 'lib/rubocop/config.rb', line 152
def for_cop(cop)
@for_cop[cop]
end
|
#for_department(department_name) ⇒ Config
Note: the 'Enabled' attribute will be present only if specified
at the department's level
194
195
196
197
|
# File 'lib/rubocop/config.rb', line 194
def for_department(department_name)
@for_department ||= Hash.new { |h, dept| h[dept] = self[dept] || {} }
@for_department[department_name.to_s]
end
|
#for_enabled_cop(cop) ⇒ Config, Hash
If the given cop is enabled, returns its configuration hash.
Otherwise, returns an empty hash.
159
160
161
|
# File 'lib/rubocop/config.rb', line 159
def for_enabled_cop(cop)
cop_enabled?(cop) ? for_cop(cop) : EMPTY_CONFIG
end
|
#gem_versions_in_target ⇒ Object
Returns target's locked gem versions (i.e. from Gemfile.lock or gems.locked)
364
365
366
|
# File 'lib/rubocop/config.rb', line 364
def gem_versions_in_target
@gem_versions_in_target ||= read_gem_versions_from_target_lockfile
end
|
#inspect ⇒ Object
376
377
378
|
# File 'lib/rubocop/config.rb', line 376
def inspect "#<#{self.class.name}:#{object_id} @loaded_path=#{loaded_path}>"
end
|
#internal? ⇒ Boolean
True if this is a config file that is shipped with RuboCop
109
110
111
112
|
# File 'lib/rubocop/config.rb', line 109
def internal?
base_config_path = File.expand_path(File.join(ConfigLoader::RUBOCOP_HOME, 'config'))
File.expand_path(loaded_path).start_with?(base_config_path)
end
|
#loaded_features ⇒ Object
#make_excludes_absolute ⇒ Object
114
115
116
117
118
119
120
121
122
123
124
|
# File 'lib/rubocop/config.rb', line 114
def make_excludes_absolute
each_key do |key|
dig(key, 'Exclude')&.map! do |exclude_elem|
if exclude_elem.is_a?(String) && !absolute?(exclude_elem)
File.expand_path(File.join(base_dir_for_path_parameters, exclude_elem))
else
exclude_elem
end
end
end
end
|
#parser_engine ⇒ Object
318
319
320
|
# File 'lib/rubocop/config.rb', line 318
def parser_engine
@parser_engine ||= for_all_cops.fetch('ParserEngine', :default).to_sym
end
|
#path_relative_to_config(path) ⇒ Object
299
300
301
|
# File 'lib/rubocop/config.rb', line 299
def path_relative_to_config(path)
relative_path(path, base_dir_for_path_parameters)
end
|
#path_sourced_gems_in_target ⇒ Object
Returns the names of the target's gems that are sourced from a local path
(i.e. path: dependencies and the project's own gem when the Gemfile
uses gemspec), whose code therefore lives in the project itself.
372
373
374
|
# File 'lib/rubocop/config.rb', line 372
def path_sourced_gems_in_target
@path_sourced_gems_in_target ||= read_path_sourced_gems_from_target_lockfile
end
|
#patterns_to_exclude ⇒ Object
295
296
297
|
# File 'lib/rubocop/config.rb', line 295
def patterns_to_exclude
for_all_cops['Exclude'] || []
end
|
#patterns_to_include ⇒ Object
291
292
293
|
# File 'lib/rubocop/config.rb', line 291
def patterns_to_include
for_all_cops['Include'] || []
end
|
#pending_cops ⇒ Object
349
350
351
352
353
354
355
356
357
358
359
360
|
# File 'lib/rubocop/config.rb', line 349
def pending_cops
keys.each_with_object([]) do |qualified_cop_name, pending_cops|
department = department_of(qualified_cop_name)
next if department && department['Enabled'] == false
cop_metadata = self[qualified_cop_name]
next unless cop_metadata['Enabled'] == 'pending'
next if new_cops_covered?(qualified_cop_name)
pending_cops << CopConfig.new(qualified_cop_name, cop_metadata)
end
end
|
#possibly_include_hidden? ⇒ Boolean
Returns true if there's a chance that an Include pattern matches hidden
files, false if that's definitely not possible.
278
279
280
281
282
283
284
|
# File 'lib/rubocop/config.rb', line 278
def possibly_include_hidden?
return @possibly_include_hidden if defined?(@possibly_include_hidden)
@possibly_include_hidden = patterns_to_include.any? do |s|
s.is_a?(Regexp) || s.start_with?('.') || s.include?('/.')
end
end
|
#preview?(options = {}) ⇒ Boolean
Whether preview behavior is on, for cops that gate an unstable change
behind it and for cops that are themselves Enabled: preview.
--preview / --no-preview win over AllCops: Preview.
232
233
234
235
236
237
|
# File 'lib/rubocop/config.rb', line 232
def preview?(options = {})
preview = options[:preview]
return preview unless preview.nil?
for_all_cops['Preview'] == true
end
|
#signature ⇒ Object
104
105
106
|
# File 'lib/rubocop/config.rb', line 104
def signature
@signature ||= Digest::SHA1.hexdigest(to_s)
end
|
#smart_loaded_path ⇒ Object
333
334
335
|
# File 'lib/rubocop/config.rb', line 333
def smart_loaded_path
PathUtil.smart_path(@loaded_path)
end
|
#string_literals_frozen_by_default? ⇒ Boolean
243
244
245
|
# File 'lib/rubocop/config.rb', line 243
def string_literals_frozen_by_default?
for_all_cops['StringLiteralsFrozenByDefault']
end
|
#target_rails_version ⇒ Object
322
323
324
325
326
327
328
329
330
331
|
# File 'lib/rubocop/config.rb', line 322
def target_rails_version
@target_rails_version ||=
if for_all_cops['TargetRailsVersion']
for_all_cops['TargetRailsVersion'].to_f
elsif target_rails_version_from_bundler_lock_file
target_rails_version_from_bundler_lock_file
else
DEFAULT_RAILS_VERSION
end
end
|
#to_s ⇒ Object
100
101
102
|
# File 'lib/rubocop/config.rb', line 100
def to_s
@to_s ||= @hash.to_s
end
|
#validate_after_resolution ⇒ Object
91
92
93
94
|
# File 'lib/rubocop/config.rb', line 91
def validate_after_resolution
@validator.validate_after_resolution
self
end
|