Class: RuboCop::Cop::Lint::BinaryOperatorWithIdenticalOperands
- Defined in:
- lib/rubocop/cop/lint/binary_operator_with_identical_operands.rb
Overview
Checks for places where binary operator has identical operands.
It covers arithmetic operators: -, /, %;
comparison operators: ==, ===, =~, >, >=, <, <=;
bitwise operators: |, ^, &;
boolean operators: &&, ||
and "spaceship" operator - <=>.
Simple arithmetic operations are allowed by this cop: +, *, **, << and >>.
Although these can be rewritten in a different way, it should not be necessary to
do so. This does not include operations such as - or / where the result will
always be the same (x - x will always be 0; x / x will always be 1), and
thus are legitimate offenses.
Constant Summary collapse
- MSG =
'Binary operator `%<op>s` has identical operands.'- ALLOWED_MATH_OPERATORS =
%i[+ * ** << >>].to_set.freeze
Constants inherited from Base
Instance Attribute Summary
Attributes inherited from Base
Instance Method Summary collapse
- #on_and(node) ⇒ Object (also: #on_or)
- #on_send(node) ⇒ Object
Methods inherited from Base
#active_support_extensions_enabled?, #add_global_offense, #add_offense, autocorrect_incompatible_with, badge, #callbacks_needed, callbacks_needed, #config_to_allow_offenses, #config_to_allow_offenses=, #cop_config, cop_name, #cop_name, department, documentation_url, exclude_from_registry, #excluded_file?, #external_dependency_checksum, inherited, #initialize, joining_forces, lint?, match?, #message, #offenses, #on_investigation_end, #on_new_investigation, #on_other_file, #parse, #ready, #relevant_file?, support_autocorrect?, support_multiple_source?, #target_rails_version, #target_ruby_version
Methods included from ExcludeLimit
Methods included from AutocorrectLogic
#autocorrect?, #autocorrect_enabled?, #autocorrect_requested?, #autocorrect_with_disable_uncorrectable?, #correctable?, #disable_uncorrectable?, #safe_autocorrect?
Methods included from IgnoredNode
#ignore_node, #ignored_node?, #part_of_ignored_node?
Methods included from Util
Constructor Details
This class inherits a constructor from RuboCop::Cop::Base
Instance Method Details
#on_and(node) ⇒ Object Also known as: on_or
61 62 63 |
# File 'lib/rubocop/cop/lint/binary_operator_with_identical_operands.rb', line 61 def on_and(node) add_offense(node, message: format(MSG, op: node.operator)) if node.lhs == node.rhs end |
#on_send(node) ⇒ Object
52 53 54 55 56 57 58 59 |
# File 'lib/rubocop/cop/lint/binary_operator_with_identical_operands.rb', line 52 def on_send(node) return unless node.binary_operation? lhs, operation, rhs = *node return if ALLOWED_MATH_OPERATORS.include?(node.method_name) add_offense(node, message: format(MSG, op: operation)) if lhs == rhs end |