Class: RuboCop::Cop::Lint::DuplicateBranch
- Defined in:
- lib/rubocop/cop/lint/duplicate_branch.rb
Overview
Checks that there are no repeated bodies
within if/unless, case-when, case-in and rescue constructs.
With IgnoreLiteralBranches: true, branches are not registered
as offenses if they return a basic literal value (string, symbol,
integer, float, rational, complex, true, false, or nil), or
return an array, hash, regexp or range that only contains one of
the above basic literal values.
With IgnoreConstantBranches: true, branches are not registered
as offenses if they return a constant value.
Constant Summary collapse
- MSG =
'Duplicate branch body detected.'
Constants inherited from Base
Instance Attribute Summary
Attributes inherited from Base
Instance Method Summary collapse
- #on_branching_statement(node) ⇒ Object (also: #on_if, #on_case, #on_case_match, #on_rescue)
Methods inherited from Base
#active_support_extensions_enabled?, #add_global_offense, #add_offense, autocorrect_incompatible_with, badge, #callbacks_needed, callbacks_needed, #config_to_allow_offenses, #config_to_allow_offenses=, #cop_config, cop_name, #cop_name, department, documentation_url, exclude_from_registry, #excluded_file?, #external_dependency_checksum, inherited, #initialize, joining_forces, lint?, match?, #message, #offenses, #on_investigation_end, #on_new_investigation, #on_other_file, #parse, #ready, #relevant_file?, support_autocorrect?, support_multiple_source?, #target_rails_version, #target_ruby_version
Methods included from ExcludeLimit
Methods included from AutocorrectLogic
#autocorrect?, #autocorrect_enabled?, #autocorrect_requested?, #autocorrect_with_disable_uncorrectable?, #correctable?, #disable_uncorrectable?, #safe_autocorrect?
Methods included from IgnoredNode
#ignore_node, #ignored_node?, #part_of_ignored_node?
Methods included from Util
Constructor Details
This class inherits a constructor from RuboCop::Cop::Base
Instance Method Details
#on_branching_statement(node) ⇒ Object Also known as: on_if, on_case, on_case_match, on_rescue
89 90 91 92 93 94 95 |
# File 'lib/rubocop/cop/lint/duplicate_branch.rb', line 89 def on_branching_statement(node) branches(node).each_with_object(Set.new) do |branch, previous| next unless consider_branch?(branch) add_offense(offense_range(branch)) unless previous.add?(branch) end end |