Class: RuboCop::Cop::Lint::RequireParentheses

Inherits:
Base
  • Object
show all
Includes:
RangeHelp
Defined in:
lib/rubocop/cop/lint/require_parentheses.rb

Overview

Checks for method calls with at least one argument where no parentheses are used around the parameter list, and the call could be misread as an operand of a boolean operator (‘&&` or `||`). Two forms are flagged:

  • a predicate method whose last argument is a ‘&&`/`||` expression, and

  • any method whose first argument is a ternary expression with a ‘&&`/`||` condition.

The idea behind warning for these constructs is that the user might be under the impression that the return value from the method call is an operand of &&/||.

Examples:


# bad
if day.is? :tuesday && month == :jan
  # ...
end

# good
if day.is?(:tuesday) && month == :jan
  # ...
end

# bad
foo a && b ? c : d

# good
foo(a && b ? c : d)

Constant Summary collapse

MSG =
'Use parentheses in the method call to avoid confusion about precedence.'

Constants included from RangeHelp

RangeHelp::BYTE_ORDER_MARK, RangeHelp::NOT_GIVEN

Constants inherited from Base

Base::RESTRICT_ON_SEND

Instance Attribute Summary

Attributes inherited from Base

#config, #processed_source, #project_index

Instance Method Summary collapse

Methods inherited from Base

#active_support_extensions_enabled?, #add_global_offense, #add_offense, #always_autocorrect?, autocorrect_incompatible_with, badge, #begin_investigation, #callbacks_needed, callbacks_needed, #config_to_allow_offenses, #config_to_allow_offenses=, #contextual_autocorrect?, #cop_config, cop_name, #cop_name, department, documentation_url, exclude_from_registry, #excluded_file?, #external_dependency_checksum, inherited, #initialize, #inspect, joining_forces, lint?, match?, #message, #offenses, #on_investigation_end, #on_new_investigation, #on_other_file, #parse, #parser_engine, #ready, #relevant_file?, requires_gem, #string_literals_frozen_by_default?, support_autocorrect?, support_multiple_source?, #target_gem_version, #target_rails_version, #target_ruby_version

Methods included from ExcludeLimit

cop_dir_for, #exclude_limit, read_limits

Methods included from AutocorrectLogic

#autocorrect?, #autocorrect_enabled?, #autocorrect_requested?, #autocorrect_with_disable_uncorrectable?, #correctable?, #disable_uncorrectable?, #safe_autocorrect?

Methods included from IgnoredNode

#ignore_node, #ignored_node?, #part_of_ignored_node?

Methods included from Util

silence_warnings

Constructor Details

This class inherits a constructor from RuboCop::Cop::Base

Instance Method Details

#on_send(node) ⇒ Object Also known as: on_csend



40
41
42
43
44
45
46
47
48
# File 'lib/rubocop/cop/lint/require_parentheses.rb', line 40

def on_send(node)
  return if !node.arguments? || node.parenthesized?

  if node.first_argument.if_type? && node.first_argument.ternary?
    check_ternary(node.first_argument, node)
  elsif node.predicate_method?
    check_predicate(node.last_argument, node)
  end
end