Class: RuboCop::Cop::Lint::UnreachablePatternBranch

Inherits:
Base
  • Object
show all
Extended by:
TargetRubyVersion
Defined in:
lib/rubocop/cop/lint/unreachable_pattern_branch.rb

Overview

Checks for unreachable in pattern branches in case...in statements.

An in branch is unreachable when a previous branch uses an unguarded catch-all pattern that matches any value unconditionally. Any in branches (and else) that follow such a catch-all are dead code.

A catch-all pattern is one of:

  • A bare variable capture (‘in x`)

  • An underscore (‘in _`)

  • A pattern alias where the left side is a catch-all (‘in _ => y`)

  • An alternation pattern where at least one alternative is a catch-all (‘in _ | Integer`)

NOTE: A catch-all pattern with a guard clause (e.g., ‘in _ if condition`) does NOT make subsequent branches unreachable because the guard might not be satisfied.

Examples:


# bad
case value
in Integer
  handle_integer
in x
  handle_other
in String
  handle_string
else
  handle_else
end

# good
case value
in Integer
  handle_integer
in String
  handle_string
in x
  handle_other
end

# bad - else is unreachable after catch-all
case value
in Integer
  handle_integer
in _
  handle_other
else
  handle_else
end

# good - guard clause means catch-all might not match
case value
in x if x.positive?
  handle_positive
in Integer
  handle_integer
else
  handle_other
end

Constant Summary collapse

MSG =
'Unreachable `in` pattern branch detected.'
MSG_ELSE =
'Unreachable `else` branch detected.'

Constants inherited from Base

Base::RESTRICT_ON_SEND

Instance Attribute Summary

Attributes inherited from Base

#config, #processed_source

Instance Method Summary collapse

Methods included from TargetRubyVersion

maximum_target_ruby_version, minimum_target_ruby_version, required_maximum_ruby_version, required_minimum_ruby_version, support_target_ruby_version?

Methods inherited from Base

#active_support_extensions_enabled?, #add_global_offense, #add_offense, #always_autocorrect?, autocorrect_incompatible_with, badge, #begin_investigation, #callbacks_needed, callbacks_needed, #config_to_allow_offenses, #config_to_allow_offenses=, #contextual_autocorrect?, #cop_config, #cop_name, cop_name, department, documentation_url, exclude_from_registry, #excluded_file?, #external_dependency_checksum, inherited, #initialize, #inspect, joining_forces, lint?, match?, #message, #offenses, #on_investigation_end, #on_new_investigation, #on_other_file, #parse, #parser_engine, #ready, #relevant_file?, requires_gem, #string_literals_frozen_by_default?, support_autocorrect?, support_multiple_source?, #target_gem_version, #target_rails_version, #target_ruby_version

Methods included from ExcludeLimit

#exclude_limit

Methods included from AutocorrectLogic

#autocorrect?, #autocorrect_enabled?, #autocorrect_requested?, #autocorrect_with_disable_uncorrectable?, #correctable?, #disable_uncorrectable?, #safe_autocorrect?

Methods included from IgnoredNode

#ignore_node, #ignored_node?, #part_of_ignored_node?

Methods included from Util

silence_warnings

Constructor Details

This class inherits a constructor from RuboCop::Cop::Base

Instance Method Details

#on_case_match(case_node) ⇒ Object



76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
# File 'lib/rubocop/cop/lint/unreachable_pattern_branch.rb', line 76

def on_case_match(case_node)
  catch_all_found = false

  case_node.in_pattern_branches.each do |in_pattern_node|
    if catch_all_found
      add_offense(in_pattern_node)
      next
    end

    pattern = in_pattern_node.pattern
    guard = in_pattern_node.children[1]

    catch_all_found = true if catch_all_pattern?(pattern) && guard.nil?
  end

  return unless catch_all_found && case_node.else?

  add_offense(case_node.loc.else, message: MSG_ELSE)
end