Class: RuboCop::Cop::Metrics::PerceivedComplexity
- Inherits:
-
CyclomaticComplexity
- Object
- Base
- CyclomaticComplexity
- RuboCop::Cop::Metrics::PerceivedComplexity
- Defined in:
- lib/rubocop/cop/metrics/perceived_complexity.rb
Overview
Tries to produce a complexity score that's a measure of the
complexity the reader experiences when looking at a method. For that
reason it considers when nodes as something that doesn't add as much
complexity as an if or a &&. Except if it's one of those special
case/when constructs where there's no expression after case. Then
the cop treats it as an if/elsif/elsif... and lets all the when
nodes count. In contrast to the CyclomaticComplexity cop, this cop
considers else nodes as adding complexity.
A case/in branch whose pattern is a simple literal (e.g. in 1, in "red", in 1..10)
or a constant/type (e.g. in Integer) and has no guard is just as easy to read as a when
branch, so it is discounted the same way. Branches with structural patterns (e.g. array,
hash, or find patterns), bindings, alternatives, or a guard add the full complexity of
a decision point.
Constant Summary collapse
- MSG =
'Perceived complexity for `%<method>s` is too high. [%<complexity>d/%<max>d]'- COUNTED_NODES =
( CyclomaticComplexity::COUNTED_NODES - %i[when in_pattern] + %i[case case_match] ).freeze
Constants inherited from Base
Instance Attribute Summary
Attributes inherited from Base
#config, #processed_source, #project_index
Method Summary
Methods included from RuboCop::Cop::MethodComplexity
Methods included from ExcludeLimit
cop_dir_for, #exclude_limit, read_limits
Methods included from Utils::RepeatedCsendDiscount
#discount_for_repeated_csend?, #reset_on_lvasgn, #reset_repeated_csend
Methods inherited from Base
#active_support_extensions_enabled?, #add_global_offense, #add_offense, #always_autocorrect?, autocorrect_incompatible_with, badge, #begin_investigation, #callbacks_needed, callbacks_needed, #config_to_allow_offenses, #config_to_allow_offenses=, #contextual_autocorrect?, #cop_config, cop_name, #cop_name, department, documentation_url, exclude_from_registry, #excluded_file?, #external_dependency_checksum, inherited, #initialize, #inspect, joining_forces, lint?, match?, #message, #offenses, #on_investigation_end, #on_new_investigation, #on_other_file, #parse, #parser_engine, #ready, #relevant_file?, requires_gem, #string_literals_frozen_by_default?, support_autocorrect?, support_multiple_source?, #target_gem_version, #target_rails_version, #target_ruby_version
Methods included from AutocorrectLogic
#autocorrect?, #autocorrect_enabled?, #autocorrect_requested?, #autocorrect_with_disable_uncorrectable?, #correctable?, #disable_uncorrectable?, #safe_autocorrect?
Methods included from IgnoredNode
#ignore_node, #ignored_node?, #part_of_ignored_node?
Methods included from Util
Constructor Details
This class inherits a constructor from RuboCop::Cop::Base