Class: RuboCop::Cop::Naming::BlockForwarding
- Extended by:
- AutoCorrector, TargetRubyVersion
- Includes:
- ConfigurableEnforcedStyle, RangeHelp
- Defined in:
- lib/rubocop/cop/naming/block_forwarding.rb
Overview
In Ruby 3.1, anonymous block forwarding has been added.
This cop identifies places where do_something(&block) can be replaced
by do_something(&).
It also supports the opposite style by alternative explicit option.
You can specify the block variable name for autocorrection with BlockForwardingName.
The default variable name is block. If the name is already in use, it will not be
autocorrected.
Constant Summary collapse
- MSG =
'Use %<style>s block forwarding.'
Constants inherited from Base
Instance Attribute Summary
Attributes inherited from Base
Instance Method Summary collapse
- #on_def(node) ⇒ Object (also: #on_defs)
Methods included from AutoCorrector
Methods included from TargetRubyVersion
minimum_target_ruby_version, required_minimum_ruby_version, support_target_ruby_version?
Methods included from ConfigurableEnforcedStyle
#alternative_style, #alternative_styles, #ambiguous_style_detected, #correct_style_detected, #detected_style, #detected_style=, #no_acceptable_style!, #no_acceptable_style?, #opposite_style_detected, #style, #style_configured?, #style_detected, #style_parameter_name, #supported_styles, #unexpected_style_detected
Methods inherited from Base
#active_support_extensions_enabled?, #add_global_offense, #add_offense, autocorrect_incompatible_with, badge, #callbacks_needed, callbacks_needed, #config_to_allow_offenses, #config_to_allow_offenses=, #cop_config, cop_name, #cop_name, department, documentation_url, exclude_from_registry, #excluded_file?, #external_dependency_checksum, inherited, #initialize, joining_forces, lint?, match?, #message, #offenses, #on_investigation_end, #on_new_investigation, #on_other_file, #parse, #ready, #relevant_file?, support_autocorrect?, support_multiple_source?, #target_rails_version, #target_ruby_version
Methods included from ExcludeLimit
Methods included from AutocorrectLogic
#autocorrect?, #autocorrect_enabled?, #autocorrect_requested?, #autocorrect_with_disable_uncorrectable?, #correctable?, #disable_uncorrectable?, #safe_autocorrect?
Methods included from IgnoredNode
#ignore_node, #ignored_node?, #part_of_ignored_node?
Methods included from Util
Constructor Details
This class inherits a constructor from RuboCop::Cop::Base
Instance Method Details
#on_def(node) ⇒ Object Also known as: on_defs
50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 |
# File 'lib/rubocop/cop/naming/block_forwarding.rb', line 50 def on_def(node) return if node.arguments.empty? last_argument = node.arguments.last return if expected_block_forwarding_style?(node, last_argument) register_offense(last_argument, node) node.each_descendant(:block_pass) do |block_pass_node| next if block_pass_node.children.first&.sym_type? || last_argument.source != block_pass_node.source register_offense(block_pass_node, node) end end |