Class: RuboCop::Cop::Style::CollectionMethods
- Extended by:
- AutoCorrector
- Includes:
- MethodPreference
- Defined in:
- lib/rubocop/cop/style/collection_methods.rb
Overview
This cop enforces the use of consistent method names from the Enumerable module.
You can customize the mapping from undesired method to desired method.
e.g. to use detect over find:
Style/CollectionMethods:
PreferredMethods:
find: detect
Constant Summary collapse
- MSG =
'Prefer `%<prefer>s` over `%<current>s`.'
Constants inherited from Base
Instance Attribute Summary
Attributes inherited from Base
Instance Method Summary collapse
Methods included from AutoCorrector
Methods inherited from Base
#add_global_offense, #add_offense, autocorrect_incompatible_with, badge, #callbacks_needed, callbacks_needed, #config_to_allow_offenses, #config_to_allow_offenses=, #cop_config, cop_name, #cop_name, department, documentation_url, exclude_from_registry, #excluded_file?, #external_dependency_checksum, inherited, #initialize, joining_forces, lint?, match?, #offenses, #on_investigation_end, #on_new_investigation, #on_other_file, #parse, #ready, #relevant_file?, support_autocorrect?, support_multiple_source?, #target_rails_version, #target_ruby_version
Methods included from ExcludeLimit
Methods included from AutocorrectLogic
#autocorrect?, #autocorrect_enabled?, #autocorrect_requested?, #correctable?, #disable_uncorrectable?, #safe_autocorrect?
Methods included from IgnoredNode
#ignore_node, #ignored_node?, #part_of_ignored_node?
Methods included from Util
Constructor Details
This class inherits a constructor from RuboCop::Cop::Base
Instance Method Details
#on_block(node) ⇒ Object
47 48 49 |
# File 'lib/rubocop/cop/style/collection_methods.rb', line 47 def on_block(node) check_method_node(node.send_node) end |
#on_send(node) ⇒ Object
51 52 53 54 55 |
# File 'lib/rubocop/cop/style/collection_methods.rb', line 51 def on_send(node) return unless implicit_block?(node) check_method_node(node) end |