Class: RuboCop::Cop::Style::EvalWithLocation

Inherits:
Cop
  • Object
show all
Defined in:
lib/rubocop/cop/style/eval_with_location.rb

Overview

This cop checks eval method usage. eval can receive source location metadata, that are filename and line number. The metadata is used by backtraces. This cop recommends to pass the metadata to eval method.

Examples:

# bad
eval "  def do_something\n  end\n"

# bad
C.class_eval "  def do_something\n  end\n"

# good
eval "  def do_something\n  end\n", binding, __FILE__, __LINE__ + 1

# good
C.class_eval "  def do_something\n  end\n", __FILE__, __LINE__ + 1

Constant Summary collapse

MSG =
'Pass `__FILE__` and `__LINE__` to `eval` method, ' \
'as they are used by backtraces.'
MSG_INCORRECT_LINE =
'Use `%<expected>s` instead of `%<actual>s`, ' \
'as they are used by backtraces.'

Constants included from Util

Util::LITERAL_REGEX

Instance Attribute Summary

Attributes inherited from Cop

#config, #corrections, #offenses, #processed_source

Instance Method Summary collapse

Methods inherited from Cop

#add_offense, all, autocorrect_incompatible_with, badge, #config_to_allow_offenses, #config_to_allow_offenses=, #cop_config, #cop_name, cop_name, #correct, department, #disable_uncorrectable, #duplicate_location?, #excluded_file?, #find_location, #highlights, inherited, #initialize, #join_force?, lint?, match?, #message, #messages, #parse, qualified_cop_name, #reason_to_not_correct, #relevant_file?, #target_rails_version, #target_ruby_version

Methods included from AST::Sexp

#s

Methods included from NodePattern::Macros

#def_node_matcher, #def_node_search, #node_search, #node_search_all, #node_search_body, #node_search_first

Methods included from AutocorrectLogic

#autocorrect?, #autocorrect_enabled?, #autocorrect_requested?, #correctable?, #disable_offense, #disable_uncorrectable?, #support_autocorrect?

Methods included from IgnoredNode

#ignore_node, #ignored_node?, #part_of_ignored_node?

Methods included from Util

begins_its_line?, comment_line?, double_quotes_required?, escape_string, first_part_of_call_chain, interpret_string_escapes, line_range, needs_escaping?, on_node, parentheses?, same_line?, to_string_literal, to_supported_styles, tokens, trim_string_interporation_escape_character

Methods included from PathUtil

absolute?, chdir, hidden_dir?, hidden_file_in_not_hidden_dir?, match_path?, pwd, relative_path, reset_pwd, smart_path

Constructor Details

This class inherits a constructor from RuboCop::Cop::Cop

Instance Method Details

#on_send(node) ⇒ Object



63
64
65
66
67
68
69
70
71
# File 'lib/rubocop/cop/style/eval_with_location.rb', line 63

def on_send(node)
  eval_without_location?(node) do |code|
    if with_lineno?(node)
      on_with_lineno(node, code)
    else
      add_offense(node)
    end
  end
end