Class: RuboCop::Cop::Style::FetchEnvVar

Inherits:
Base
  • Object
show all
Extended by:
AutoCorrector
Defined in:
lib/rubocop/cop/style/fetch_env_var.rb

Overview

Suggests ENV.fetch for the replacement of ENV[]. ENV[] silently fails and returns nil when the environment variable is unset, which may cause unexpected behaviors when the developer forgets to set it. On the other hand, ENV.fetch raises KeyError or returns the explicitly specified default value.

Examples:

DefaultToNil: true (default)

# bad
ENV['X']
x = ENV['X']

# good
ENV.fetch('X', nil)
x = ENV.fetch('X', nil)

# also good
!ENV['X']
ENV['X'].some_method # (e.g. `.nil?`)

DefaultToNil: false

# bad
ENV['X']
x = ENV['X']

# good
ENV.fetch('X')
x = ENV.fetch('X')

# also good
!ENV['X']
ENV['X'].some_method # (e.g. `.nil?`)

Constant Summary collapse

MSG_WITH_NIL =
'Use `ENV.fetch(%<key>s, nil)` instead of `ENV[%<key>s]`.'
MSG_WITHOUT_NIL =
'Use `ENV.fetch(%<key>s)` instead of `ENV[%<key>s]`.'
RESTRICT_ON_SEND =
[:[]].freeze

Instance Attribute Summary

Attributes inherited from Base

#config, #processed_source, #project_index

Instance Method Summary collapse

Methods included from AutoCorrector

support_autocorrect?

Methods inherited from Base

#active_support_extensions_enabled?, #add_global_offense, #add_offense, #always_autocorrect?, autocorrect_incompatible_with, badge, #begin_investigation, #callbacks_needed, callbacks_needed, #config_to_allow_offenses, #config_to_allow_offenses=, #contextual_autocorrect?, #cop_config, cop_name, #cop_name, department, documentation_url, exclude_from_registry, #excluded_file?, #external_dependency_checksum, inherited, #initialize, #inspect, joining_forces, lint?, match?, #message, #offenses, #on_investigation_end, #on_new_investigation, #on_other_file, #parse, #parser_engine, #ready, #relevant_file?, requires_gem, #string_literals_frozen_by_default?, support_autocorrect?, support_multiple_source?, #target_gem_version, #target_rails_version, #target_ruby_version

Methods included from ExcludeLimit

cop_dir_for, #exclude_limit, read_limits

Methods included from AutocorrectLogic

#autocorrect?, #autocorrect_enabled?, #autocorrect_requested?, #autocorrect_with_disable_uncorrectable?, #correctable?, #disable_uncorrectable?, #safe_autocorrect?

Methods included from IgnoredNode

#ignore_node, #ignored_node?, #part_of_ignored_node?

Methods included from Util

silence_warnings

Constructor Details

This class inherits a constructor from RuboCop::Cop::Base

Instance Method Details

#env_with_bracket?(node) ⇒ Object



46
47
48
# File 'lib/rubocop/cop/style/fetch_env_var.rb', line 46

def_node_matcher :env_with_bracket?, <<~PATTERN
  (send (const nil? :ENV) :[] $_)
PATTERN

#on_send(node) ⇒ Object



50
51
52
53
54
55
56
57
58
59
# File 'lib/rubocop/cop/style/fetch_env_var.rb', line 50

def on_send(node)
  env_with_bracket?(node) do |name_node|
    break unless offensive?(node)

    message = format(offense_message, key: name_node.source)
    add_offense(node, message: message) do |corrector|
      corrector.replace(node, new_code(name_node))
    end
  end
end