Class: RuboCop::Cop::Style::ParallelAssignment::RescueCorrector
- Inherits:
-
GenericCorrector
- Object
- GenericCorrector
- RuboCop::Cop::Style::ParallelAssignment::RescueCorrector
- Defined in:
- lib/rubocop/cop/style/parallel_assignment.rb
Overview
An internal class for correcting parallel assignment protected by rescue
Constant Summary
Constants included from Alignment
Instance Attribute Summary
Attributes inherited from GenericCorrector
#config, #node, #rescue_result, #rhs
Instance Method Summary collapse
Methods inherited from GenericCorrector
Constructor Details
This class inherits a constructor from RuboCop::Cop::Style::ParallelAssignment::GenericCorrector
Instance Method Details
#correction ⇒ Object
265 266 267 268 269 270 271 272 273 274 |
# File 'lib/rubocop/cop/style/parallel_assignment.rb', line 265 def correction # If the parallel assignment uses a rescue modifier and it is the # only contents of a method, then we want to make use of the # implicit begin if rhs.parent.parent.parent&.def_type? super + def_correction(rescue_result) else begin_correction(rescue_result) end end |
#correction_range ⇒ Object
276 277 278 |
# File 'lib/rubocop/cop/style/parallel_assignment.rb', line 276 def correction_range rhs.parent.parent.source_range end |