Class: RuboCop::Cop::Style::PercentLiteralDelimiters
- Extended by:
- AutoCorrector
- Includes:
- PercentLiteral
- Defined in:
- lib/rubocop/cop/style/percent_literal_delimiters.rb
Overview
Enforces the consistent usage of ‘%`-literal delimiters. Using consistent delimiters across the codebase reduces cognitive load when reading `%`-literals.
Specify the ‘default’ key to set all preferred delimiters at once. You can continue to specify individual preferred delimiters to override the default.
Constant Summary
Constants included from RangeHelp
RangeHelp::BYTE_ORDER_MARK, RangeHelp::NOT_GIVEN
Constants inherited from Base
Instance Attribute Summary
Attributes inherited from Base
Instance Method Summary collapse
- #on_array(node) ⇒ Object
- #on_regexp(node) ⇒ Object
- #on_str(node) ⇒ Object (also: #on_dstr)
- #on_sym(node) ⇒ Object
- #on_xstr(node) ⇒ Object
Methods included from AutoCorrector
Methods inherited from Base
#active_support_extensions_enabled?, #add_global_offense, #add_offense, #always_autocorrect?, autocorrect_incompatible_with, badge, #begin_investigation, #callbacks_needed, callbacks_needed, #config_to_allow_offenses, #config_to_allow_offenses=, #contextual_autocorrect?, #cop_config, cop_name, #cop_name, department, documentation_url, exclude_from_registry, #excluded_file?, #external_dependency_checksum, inherited, #initialize, #inspect, joining_forces, lint?, match?, #offenses, #on_investigation_end, #on_new_investigation, #on_other_file, #parse, #parser_engine, #ready, #relevant_file?, requires_gem, #string_literals_frozen_by_default?, support_autocorrect?, support_multiple_source?, #target_gem_version, #target_rails_version, #target_ruby_version
Methods included from ExcludeLimit
Methods included from AutocorrectLogic
#autocorrect?, #autocorrect_enabled?, #autocorrect_requested?, #autocorrect_with_disable_uncorrectable?, #correctable?, #disable_uncorrectable?, #safe_autocorrect?
Methods included from IgnoredNode
#ignore_node, #ignored_node?, #part_of_ignored_node?
Methods included from Util
Constructor Details
This class inherits a constructor from RuboCop::Cop::Base
Instance Method Details
#on_array(node) ⇒ Object
32 33 34 |
# File 'lib/rubocop/cop/style/percent_literal_delimiters.rb', line 32 def on_array(node) process(node, '%w', '%W', '%i', '%I') end |
#on_regexp(node) ⇒ Object
36 37 38 |
# File 'lib/rubocop/cop/style/percent_literal_delimiters.rb', line 36 def on_regexp(node) process(node, '%r') end |
#on_str(node) ⇒ Object Also known as: on_dstr
40 41 42 |
# File 'lib/rubocop/cop/style/percent_literal_delimiters.rb', line 40 def on_str(node) process(node, '%', '%Q', '%q') end |
#on_sym(node) ⇒ Object
45 46 47 |
# File 'lib/rubocop/cop/style/percent_literal_delimiters.rb', line 45 def on_sym(node) process(node, '%s') end |
#on_xstr(node) ⇒ Object
49 50 51 |
# File 'lib/rubocop/cop/style/percent_literal_delimiters.rb', line 49 def on_xstr(node) process(node, '%x') end |