Class: RuboCop::Cop::Style::RedundantDoubleSplatHashBraces
- Extended by:
- AutoCorrector
- Defined in:
- lib/rubocop/cop/style/redundant_double_splat_hash_braces.rb
Overview
Checks for redundant uses of double splat hash braces.
Constant Summary collapse
- MSG =
'Remove the redundant double splat and braces, use keyword arguments directly.'
Constants inherited from Base
Instance Attribute Summary
Attributes inherited from Base
Instance Method Summary collapse
Methods included from AutoCorrector
Methods inherited from Base
#active_support_extensions_enabled?, #add_global_offense, #add_offense, autocorrect_incompatible_with, badge, #callbacks_needed, callbacks_needed, #config_to_allow_offenses, #config_to_allow_offenses=, #cop_config, cop_name, #cop_name, department, documentation_url, exclude_from_registry, #excluded_file?, #external_dependency_checksum, inherited, #initialize, joining_forces, lint?, match?, #message, #offenses, #on_investigation_end, #on_new_investigation, #on_other_file, #parse, #ready, #relevant_file?, support_autocorrect?, support_multiple_source?, #target_rails_version, #target_ruby_version
Methods included from ExcludeLimit
Methods included from AutocorrectLogic
#autocorrect?, #autocorrect_enabled?, #autocorrect_requested?, #autocorrect_with_disable_uncorrectable?, #correctable?, #disable_uncorrectable?, #safe_autocorrect?
Methods included from IgnoredNode
#ignore_node, #ignored_node?, #part_of_ignored_node?
Methods included from Util
Constructor Details
This class inherits a constructor from RuboCop::Cop::Base
Instance Method Details
#double_splat_hash_braces?(node) ⇒ Object
22 23 24 |
# File 'lib/rubocop/cop/style/redundant_double_splat_hash_braces.rb', line 22 def_node_matcher :double_splat_hash_braces?, "(hash (kwsplat (hash ...)))\n" |
#on_hash(node) ⇒ Object
26 27 28 29 30 31 32 33 34 35 |
# File 'lib/rubocop/cop/style/redundant_double_splat_hash_braces.rb', line 26 def on_hash(node) return if node.pairs.empty? || node.pairs.any?(&:hash_rocket?) grandparent = node.parent&.parent return unless double_splat_hash_braces?(grandparent) add_offense(grandparent) do |corrector| corrector.replace(grandparent, node.children.map(&:source).join(', ')) end end |