Class: RuboCop::Cop::Style::SingleLineDoEndBlock

Inherits:
Base
  • Object
show all
Extended by:
AutoCorrector
Includes:
CheckSingleLineSuitability
Defined in:
lib/rubocop/cop/style/single_line_do_end_block.rb

Overview

Checks for single-line do...end block.

In practice a single line do...end is autocorrected when EnforcedStyle: semantic is configured for Style/BlockDelimiters. The autocorrection maintains the do ... end syntax to preserve semantics and does not change it to {`...`} block.

NOTE: If InspectBlocks is set to true for Layout/RedundantLineBreak, blocks will be autocorrected to be on a single line if possible. This cop respects that configuration by not registering an offense if it would subsequently cause a Layout/RedundantLineBreak offense.

Examples:


# bad
foo do |arg| bar(arg) end

# good
foo do |arg|
  bar(arg)
end

# bad
->(arg) do bar(arg) end

# good
->(arg) { bar(arg) }

Constant Summary collapse

MSG =
'Prefer multiline `do`...`end` block.'

Constants inherited from Base

Base::RESTRICT_ON_SEND

Instance Attribute Summary

Attributes inherited from Base

#config, #processed_source, #project_index

Instance Method Summary collapse

Methods included from AutoCorrector

support_autocorrect?

Methods included from CheckSingleLineSuitability

#suitable_as_single_line?

Methods inherited from Base

#active_support_extensions_enabled?, #add_global_offense, #add_offense, #always_autocorrect?, autocorrect_incompatible_with, badge, #begin_investigation, #callbacks_needed, callbacks_needed, #config_to_allow_offenses, #config_to_allow_offenses=, #contextual_autocorrect?, #cop_config, cop_name, #cop_name, department, documentation_url, exclude_from_registry, #excluded_file?, #external_dependency_checksum, inherited, #initialize, #inspect, joining_forces, lint?, match?, #message, #offenses, #on_investigation_end, #on_new_investigation, #on_other_file, #parse, #parser_engine, #ready, #relevant_file?, requires_gem, #string_literals_frozen_by_default?, support_autocorrect?, support_multiple_source?, #target_gem_version, #target_rails_version, #target_ruby_version

Methods included from ExcludeLimit

cop_dir_for, #exclude_limit, read_limits

Methods included from AutocorrectLogic

#autocorrect?, #autocorrect_enabled?, #autocorrect_requested?, #autocorrect_with_disable_uncorrectable?, #correctable?, #disable_uncorrectable?, #safe_autocorrect?

Methods included from IgnoredNode

#ignore_node, #ignored_node?, #part_of_ignored_node?

Methods included from Util

silence_warnings

Constructor Details

This class inherits a constructor from RuboCop::Cop::Base

Instance Method Details

#on_block(node) ⇒ Object Also known as: on_numblock, on_itblock

rubocop:disable Metrics/AbcSize



40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
# File 'lib/rubocop/cop/style/single_line_do_end_block.rb', line 40

def on_block(node)
  return if node.multiline? || node.braces?
  return if single_line_blocks_preferred? && suitable_as_single_line?(node)

  add_offense(node) do |corrector|
    corrector.insert_after(do_line(node), "\n")

    if (heredoc = trailing_heredoc(node.body))
      # The heredoc body extends past the `end` on the source, so the
      # `end` has to be moved after it rather than before, which would
      # otherwise move it into the heredoc body and break the syntax.
      corrector.remove(node.loc.end)
      corrector.insert_after(heredoc.loc.heredoc_end, "\nend")
    else
      corrector.insert_before(node.loc.end, "\n")
    end
  end
end