Class: RuboCop::Cop::Style::TernaryParentheses

Inherits:
Cop
  • Object
show all
Includes:
ConfigurableEnforcedStyle, RuboCop::Cop::SafeAssignment, RuboCop::Cop::SurroundingSpace
Defined in:
lib/rubocop/cop/style/ternary_parentheses.rb

Overview

This cop checks for the presence of parentheses around ternary conditions. It is configurable to enforce inclusion or omission of parentheses using EnforcedStyle. Omission is only enforced when removing the parentheses won’t cause a different behavior.

AllowSafeAssignment option for safe assignment. By safe assignment we mean putting parentheses around an assignment to indicate “I know I’m using an assignment as a condition. It’s not a mistake.”

Examples:

EnforcedStyle: require_no_parentheses (default)

# bad
foo = (bar?) ? a : b
foo = (bar.baz?) ? a : b
foo = (bar && baz) ? a : b

# good
foo = bar? ? a : b
foo = bar.baz? ? a : b
foo = bar && baz ? a : b

EnforcedStyle: require_parentheses

# bad
foo = bar? ? a : b
foo = bar.baz? ? a : b
foo = bar && baz ? a : b

# good
foo = (bar?) ? a : b
foo = (bar.baz?) ? a : b
foo = (bar && baz) ? a : b

EnforcedStyle: require_parentheses_when_complex

# bad
foo = (bar?) ? a : b
foo = (bar.baz?) ? a : b
foo = bar && baz ? a : b

# good
foo = bar? ? a : b
foo = bar.baz? ? a : b
foo = (bar && baz) ? a : b

AllowSafeAssignment: true (default)

# good
foo = (bar = baz) ? a : b

AllowSafeAssignment: false

# bad
foo = (bar = baz) ? a : b

Constant Summary collapse

VARIABLE_TYPES =
AST::Node::VARIABLES
NON_COMPLEX_TYPES =
[*VARIABLE_TYPES, :const, :defined?, :yield].freeze
MSG =
'%<command>s parentheses for ternary conditions.'
MSG_COMPLEX =
'%<command>s parentheses for ternary expressions with' \
' complex conditions.'

Constants included from RuboCop::Cop::SurroundingSpace

RuboCop::Cop::SurroundingSpace::NO_SPACE_COMMAND, RuboCop::Cop::SurroundingSpace::SINGLE_SPACE_REGEXP, RuboCop::Cop::SurroundingSpace::SPACE_COMMAND

Constants included from Util

Util::LITERAL_REGEX

Instance Attribute Summary

Attributes inherited from Cop

#config, #corrections, #offenses, #processed_source

Instance Method Summary collapse

Methods included from ConfigurableEnforcedStyle

#alternative_style, #alternative_styles, #ambiguous_style_detected, #correct_style_detected, #detected_style, #detected_style=, #no_acceptable_style!, #no_acceptable_style?, #opposite_style_detected, #style, #style_detected, #style_parameter_name, #supported_styles, #unexpected_style_detected

Methods included from NodePattern::Macros

#def_node_matcher, #def_node_search, #node_search, #node_search_all, #node_search_body, #node_search_first

Methods inherited from Cop

#add_offense, all, autocorrect_incompatible_with, badge, #config_to_allow_offenses, #config_to_allow_offenses=, #cop_config, #cop_name, cop_name, #correct, department, #disable_uncorrectable, #duplicate_location?, #excluded_file?, #find_location, #highlights, inherited, #initialize, #join_force?, lint?, match?, #messages, #parse, qualified_cop_name, #reason_to_not_correct, #relevant_file?, #target_rails_version, #target_ruby_version

Methods included from AST::Sexp

#s

Methods included from AutocorrectLogic

#autocorrect?, #autocorrect_enabled?, #autocorrect_requested?, #correctable?, #disable_offense, #disable_uncorrectable?, #support_autocorrect?

Methods included from IgnoredNode

#ignore_node, #ignored_node?, #part_of_ignored_node?

Methods included from Util

begins_its_line?, comment_line?, double_quotes_required?, escape_string, first_part_of_call_chain, interpret_string_escapes, line_range, needs_escaping?, on_node, parentheses?, same_line?, to_string_literal, to_supported_styles, tokens, trim_string_interporation_escape_character

Methods included from PathUtil

absolute?, chdir, hidden_dir?, hidden_file_in_not_hidden_dir?, match_path?, pwd, relative_path, reset_pwd, smart_path

Constructor Details

This class inherits a constructor from RuboCop::Cop::Cop

Instance Method Details

#autocorrect(node) ⇒ Object



80
81
82
83
84
85
86
87
88
89
90
91
92
# File 'lib/rubocop/cop/style/ternary_parentheses.rb', line 80

def autocorrect(node)
  condition = node.condition

  return nil if parenthesized?(condition) &&
                (safe_assignment?(condition) ||
                unsafe_autocorrect?(condition))

  if parenthesized?(condition)
    correct_parenthesized(condition)
  else
    correct_unparenthesized(condition)
  end
end

#on_if(node) ⇒ Object



69
70
71
72
73
74
# File 'lib/rubocop/cop/style/ternary_parentheses.rb', line 69

def on_if(node)
  return if only_closing_parenthesis_is_last_line?(node.condition)
  return unless node.ternary? && !infinite_loop? && offense?(node)

  add_offense(node, location: node.source_range)
end

#only_closing_parenthesis_is_last_line?(condition) ⇒ Boolean

Returns:

  • (Boolean)


76
77
78
# File 'lib/rubocop/cop/style/ternary_parentheses.rb', line 76

def only_closing_parenthesis_is_last_line?(condition)
  condition.source.split("\n").last == ')'
end