Class: Rubocop::Cop::Style::Alias
- Defined in:
- lib/rubocop/cop/style/alias.rb
Overview
The purpose of the this cop is advise the use of alias_method over the alias keyword whenever possible.
Constant Summary collapse
- MSG =
'Use alias_method instead of alias.'
Constants inherited from Cop
Constants included from Util
Instance Attribute Summary
Attributes inherited from Cop
#config, #corrections, #offences, #processed_source
Instance Method Summary collapse
Methods inherited from Cop
#add_offence, all, #autocorrect?, #config_to_allow_offences, #config_to_allow_offences=, #cop_config, cop_name, #cop_name, cop_type, #debug?, #exclude_file?, #exclude_paths, #ignore_node, #include_file?, #include_paths, inherited, #initialize, lint?, #message, non_rails, rails?, #relevant_file?, style?, #support_autocorrect?
Methods included from Util
block_length, command?, comment_line?, const_name, first_part_of_call_chain, lambda?, lambda_or_proc?, line_range, on_node, proc?, range_with_surrounding_space, source_range, strip_quotes
Constructor Details
This class inherits a constructor from Rubocop::Cop::Cop
Instance Method Details
#autocorrect(node) ⇒ Object
33 34 35 36 37 38 39 40 41 42 43 44 |
# File 'lib/rubocop/cop/style/alias.rb', line 33 def autocorrect(node) @corrections << lambda do |corrector| # replace alias with alias_method corrector.replace(node.loc.keyword, 'alias_method') # insert a comma new, old = *node corrector.insert_after(new.loc.expression, ',') # convert bareword arguments to symbols corrector.replace(new.loc.expression, ":#{new.children.first}") corrector.replace(old.loc.expression, ":#{old.children.first}") end end |
#on_alias(node) ⇒ Object
22 23 24 25 26 27 28 29 30 31 |
# File 'lib/rubocop/cop/style/alias.rb', line 22 def on_alias(node) return if ignored_node?(node) # alias_method can't be used with global variables new, old = *node return if new.type == :gvar && old.type == :gvar add_offence(node, :keyword) end |
#on_block(node) ⇒ Object
11 12 13 14 15 16 17 18 19 20 |
# File 'lib/rubocop/cop/style/alias.rb', line 11 def on_block(node) method, _args, body = *node _receiver, method_name = *method # using alias is the only option in certain scenarios # in such scenarios we don't want to report an offence if method_name == :instance_exec on_node(:alias, body) { |n| ignore_node(n) } end end |