Class: Rubocop::Cop::Style::ConstantName

Inherits:
Cop
  • Object
show all
Defined in:
lib/rubocop/cop/style/constant_name.rb

Overview

This cop checks whether constant names are written using SCREAMING_SNAKE_CASE.

To avoid false positives, it ignores cases in which we cannot know for certain the type of value that would be assigned to a constant.

Constant Summary collapse

MSG =
'Use SCREAMING_SNAKE_CASE for constants.'
SNAKE_CASE =
/^[\dA-Z_]+$/

Constants inherited from Cop

Cop::OPERATOR_METHODS

Constants included from Util

Util::PROC_NEW_NODE

Instance Attribute Summary

Attributes inherited from Cop

#config, #corrections, #offences, #processed_source

Instance Method Summary collapse

Methods inherited from Cop

#add_offence, all, #autocorrect?, #config_to_allow_offences, #config_to_allow_offences=, #cop_config, cop_name, #cop_name, cop_type, #debug?, #exclude_file?, #exclude_paths, #ignore_node, #include_file?, #include_paths, inherited, #initialize, lint?, #message, non_rails, rails?, #relevant_file?, style?, #support_autocorrect?

Methods included from Util

block_length, command?, comment_line?, const_name, first_part_of_call_chain, lambda?, lambda_or_proc?, line_range, on_node, proc?, range_with_surrounding_space, source_range, strip_quotes

Constructor Details

This class inherits a constructor from Rubocop::Cop::Cop

Instance Method Details

#on_casgn(node) ⇒ Object



15
16
17
18
19
20
21
22
23
24
25
# File 'lib/rubocop/cop/style/constant_name.rb', line 15

def on_casgn(node)
  _scope, const_name, value = *node

  # We cannot know the result of method calls like
  # NewClass = something_that_returns_a_class
  # It's also ok to assign a class constant another class constant
  # SomeClass = SomeOtherClass
  unless value && [:send, :block, :const].include?(value.type)
    add_offence(node, :name) if const_name !~ SNAKE_CASE
  end
end