Class: Rubocop::Cop::Style::EmptyLiteral
- Defined in:
- lib/rubocop/cop/style/empty_literal.rb
Overview
This cop checks for the use of a method, the result of which would be a literal, like an empty array, hash or string.
Constant Summary collapse
- ARR_MSG =
'Use array literal [] instead of Array.new.'- HASH_MSG =
'Use hash literal {} instead of Hash.new.'- STR_MSG =
"Use string literal '' instead of String.new."- ARRAY_NODE =
Empty array node
(send (const nil :Array) :new)
s(:send, s(:const, nil, :Array), :new)
- HASH_NODE =
Empty hash node
(send (const nil :Hash) :new)
s(:send, s(:const, nil, :Hash), :new)
- STR_NODE =
Empty string node
(send (const nil :String) :new)
s(:send, s(:const, nil, :String), :new)
Constants inherited from Cop
Constants included from Util
Instance Attribute Summary
Attributes inherited from Cop
#config, #corrections, #offences, #processed_source
Instance Method Summary collapse
Methods inherited from Cop
#add_offence, all, #autocorrect?, #config_to_allow_offences, #config_to_allow_offences=, #cop_config, cop_name, #cop_name, cop_type, #debug?, #exclude_file?, #exclude_paths, #ignore_node, #include_file?, #include_paths, inherited, #initialize, lint?, #message, non_rails, rails?, #relevant_file?, style?, #support_autocorrect?
Methods included from Util
block_length, command?, comment_line?, const_name, first_part_of_call_chain, lambda?, lambda_or_proc?, line_range, on_node, proc?, range_with_surrounding_space, source_range, strip_quotes
Constructor Details
This class inherits a constructor from Rubocop::Cop::Cop
Instance Method Details
#autocorrect(node) ⇒ Object
47 48 49 50 51 52 53 54 55 56 |
# File 'lib/rubocop/cop/style/empty_literal.rb', line 47 def autocorrect(node) @corrections << lambda do |corrector| name = case node when ARRAY_NODE then '[]' when HASH_NODE then '{}' when STR_NODE then "''" end corrector.replace(node.loc.expression, name) end end |
#on_send(node) ⇒ Object
31 32 33 34 35 36 37 38 39 40 41 42 |
# File 'lib/rubocop/cop/style/empty_literal.rb', line 31 def on_send(node) return if part_of_ignored_node?(node) case node when ARRAY_NODE add_offence(node, :expression, ARR_MSG) when HASH_NODE add_offence(node, :expression, HASH_MSG) when STR_NODE add_offence(node, :expression, STR_MSG) end end |