Class: Rubocop::Cop::Style::EndOfLine

Inherits:
Cop
  • Object
show all
Defined in:
lib/rubocop/cop/style/end_of_line.rb

Overview

This cop checks for Windows-style line endings in the source code.

Constant Summary collapse

MSG =
'Carriage return character detected.'

Constants inherited from Cop

Cop::OPERATOR_METHODS

Constants included from Util

Util::PROC_NEW_NODE

Instance Attribute Summary

Attributes inherited from Cop

#config, #corrections, #offences, #processed_source

Instance Method Summary collapse

Methods inherited from Cop

#add_offence, all, #autocorrect?, #config_to_allow_offences, #config_to_allow_offences=, #cop_config, cop_name, #cop_name, cop_type, #debug?, #exclude_file?, #exclude_paths, #ignore_node, #include_file?, #include_paths, inherited, #initialize, lint?, #message, non_rails, rails?, #relevant_file?, style?, #support_autocorrect?

Methods included from Util

block_length, command?, comment_line?, const_name, first_part_of_call_chain, lambda?, lambda_or_proc?, line_range, on_node, proc?, range_with_surrounding_space, source_range, strip_quotes

Constructor Details

This class inherits a constructor from Rubocop::Cop::Cop

Instance Method Details

#investigate(processed_source) ⇒ Object



10
11
12
13
14
15
16
17
18
19
20
21
22
23
24
25
26
# File 'lib/rubocop/cop/style/end_of_line.rb', line 10

def investigate(processed_source)
  buffer = processed_source.buffer
  original_source = IO.read(buffer.name,
                            encoding: buffer.source.encoding)
  original_source.lines.each_with_index do |line, index|
    if line =~ /\r$/
      add_offence(nil,
                  source_range(buffer,
                               processed_source[0...index],
                               0, line.length),
                  MSG)
      # Usually there will be carriage return characters on all or none
      # of the lines in a file, so we report only one offence.
      break
    end
  end
end