Class: Rubocop::Cop::Style::ParenthesesAroundCondition

Inherits:
Cop
  • Object
show all
Defined in:
lib/rubocop/cop/style/parentheses_around_condition.rb

Overview

This cop checks for the presence of superfluous parentheses around the condition of if/while/until.

Constant Summary collapse

ASGN_NODES =
[:lvasgn, :ivasgn, :cvasgn, :gvasgn, :casgn]
MSG =
"Don't use parentheses around the condition of an " +
'if/unless/while/until'

Constants inherited from Cop

Cop::OPERATOR_METHODS

Constants included from Util

Util::PROC_NEW_NODE

Instance Attribute Summary

Attributes inherited from Cop

#config, #corrections, #offences, #processed_source

Instance Method Summary collapse

Methods inherited from Cop

#add_offence, all, #autocorrect?, #config_to_allow_offences, #config_to_allow_offences=, #cop_config, cop_name, #cop_name, cop_type, #debug?, #exclude_file?, #exclude_paths, #ignore_node, #include_file?, #include_paths, inherited, #initialize, lint?, #message, non_rails, rails?, #relevant_file?, style?, #support_autocorrect?

Methods included from Util

block_length, command?, comment_line?, const_name, first_part_of_call_chain, lambda?, lambda_or_proc?, line_range, on_node, proc?, range_with_surrounding_space, source_range, strip_quotes

Constructor Details

This class inherits a constructor from Rubocop::Cop::Cop

Instance Method Details

#on_if(node) ⇒ Object



13
14
15
# File 'lib/rubocop/cop/style/parentheses_around_condition.rb', line 13

def on_if(node)
  process_control_op(node)
end

#on_until(node) ⇒ Object



21
22
23
# File 'lib/rubocop/cop/style/parentheses_around_condition.rb', line 21

def on_until(node)
  process_control_op(node)
end

#on_while(node) ⇒ Object



17
18
19
# File 'lib/rubocop/cop/style/parentheses_around_condition.rb', line 17

def on_while(node)
  process_control_op(node)
end