Class: Rubocop::Cop::Style::SpaceAfterMethodName

Inherits:
Cop
  • Object
show all
Includes:
CheckMethods
Defined in:
lib/rubocop/cop/style/space_after_method_name.rb

Overview

Checks for space between a method name and a left parenthesis.

Constant Summary collapse

MSG =
'Never put a space between a method name and the opening ' +
'parenthesis.'

Constants inherited from Cop

Cop::OPERATOR_METHODS

Constants included from Util

Util::PROC_NEW_NODE

Instance Attribute Summary

Attributes inherited from Cop

#config, #corrections, #offences, #processed_source

Instance Method Summary collapse

Methods included from CheckMethods

#on_def, #on_defs

Methods inherited from Cop

#add_offence, all, #autocorrect?, #config_to_allow_offences, #config_to_allow_offences=, #cop_config, cop_name, #cop_name, cop_type, #debug?, #exclude_file?, #exclude_paths, #ignore_node, #include_file?, #include_paths, inherited, #initialize, lint?, #message, non_rails, rails?, #relevant_file?, style?, #support_autocorrect?

Methods included from Util

block_length, command?, comment_line?, const_name, first_part_of_call_chain, lambda?, lambda_or_proc?, line_range, on_node, proc?, range_with_surrounding_space, source_range, strip_quotes

Constructor Details

This class inherits a constructor from Rubocop::Cop::Cop

Instance Method Details

#autocorrect(pos_before_left_paren) ⇒ Object



24
25
26
27
28
# File 'lib/rubocop/cop/style/space_after_method_name.rb', line 24

def autocorrect(pos_before_left_paren)
  @corrections << lambda do |corrector|
    corrector.remove(pos_before_left_paren)
  end
end

#check(_node, _method_name, args, body) ⇒ Object



13
14
15
16
17
18
19
20
21
22
# File 'lib/rubocop/cop/style/space_after_method_name.rb', line 13

def check(_node, _method_name, args, body)
  return unless args.loc.begin && args.loc.begin.is?('(')
  expr = args.loc.expression
  pos_before_left_paren = Parser::Source::Range.new(expr.source_buffer,
                                                    expr.begin_pos - 1,
                                                    expr.begin_pos)
  if pos_before_left_paren.source =~ /\s/
    add_offence(pos_before_left_paren, pos_before_left_paren)
  end
end