Class: Rubocop::Cop::Style::SpaceAroundBlockBraces

Inherits:
Cop
  • Object
show all
Includes:
ConfigurableEnforcedStyle, SurroundingSpace
Defined in:
lib/rubocop/cop/style/space_around_block_braces.rb

Overview

Checks that block braces have or don't have surrounding space depending on configuration. For blocks taking parameters, it checks that the left brace has or doesn't have trailing space depending on configuration. Also checks that the left brace is preceded by a space and this is not configurable.

Constant Summary

Constants inherited from Cop

Cop::OPERATOR_METHODS

Constants included from Util

Util::PROC_NEW_NODE

Instance Attribute Summary

Attributes inherited from Cop

#config, #corrections, #offences, #processed_source

Instance Method Summary collapse

Methods included from SurroundingSpace

#index_of_first_token, #index_of_last_token, #space_between?, #token_table

Methods included from ConfigurableEnforcedStyle

#alternative_style, #both_styles_detected, #correct_style_detected, #opposite_style_detected, #style, #unrecognized_style_detected

Methods inherited from Cop

#add_offence, all, #autocorrect?, #config_to_allow_offences, #config_to_allow_offences=, #cop_config, cop_name, #cop_name, cop_type, #debug?, #exclude_file?, #exclude_paths, #ignore_node, #include_file?, #include_paths, inherited, #initialize, lint?, #message, non_rails, rails?, #relevant_file?, style?, #support_autocorrect?

Methods included from Util

block_length, command?, comment_line?, const_name, first_part_of_call_chain, lambda?, lambda_or_proc?, line_range, on_node, proc?, range_with_surrounding_space, source_range, strip_quotes

Constructor Details

This class inherits a constructor from Rubocop::Cop::Cop

Instance Method Details

#on_block(node) ⇒ Object



15
16
17
18
19
20
21
22
23
24
25
26
27
28
29
30
# File 'lib/rubocop/cop/style/space_around_block_braces.rb', line 15

def on_block(node)
  return if node.loc.begin.is?('do') # No braces.

  # If braces are on separate lines, and the Blocks cop is enabled,
  # those braces will be changed to do..end by the user or by
  # auto-correct, so reporting space issues is not useful, and it
  # creates auto-correct conflicts.
  if config.for_cop('Blocks')['Enabled'] && Util.block_length(node) > 0
    return
  end

  left_brace, right_brace = node.loc.begin, node.loc.end

  check_outside(left_brace)
  check_inside(node, left_brace, right_brace)
end