Class: RuboCop::ConfigLoader
- Inherits:
-
Object
- Object
- RuboCop::ConfigLoader
- Extended by:
- FileFinder
- Defined in:
- lib/rubocop/config_loader.rb
Overview
This class represents the configuration of the RuboCop application and all its cops. A Config is associated with a YAML configuration file from which it was read. Several different Configs can be used during a run of the rubocop program, if files in several directories are inspected.
Constant Summary collapse
- DOTFILE =
ConfigFinder::DOTFILE
- RUBOCOP_HOME =
File.realpath(File.join(File.dirname(__FILE__), '..', '..'))
- DEFAULT_FILE =
File.join(RUBOCOP_HOME, 'config', 'default.yml')
Class Attribute Summary collapse
- .cache_root(cache_root_override = nil) ⇒ Object
-
.debug ⇒ Object
(also: debug?)
Returns the value of attribute debug.
- .default_configuration ⇒ Object
-
.disable_pending_cops ⇒ Object
Returns the value of attribute disable_pending_cops.
-
.disabled_by_default ⇒ Object
Returns the value of attribute disabled_by_default.
-
.enable_pending_cops ⇒ Object
Returns the value of attribute enable_pending_cops.
-
.enabled_by_default ⇒ Object
Returns the value of attribute enabled_by_default.
-
.ignore_parent_exclusion ⇒ Object
(also: ignore_parent_exclusion?)
Returns the value of attribute ignore_parent_exclusion.
-
.ignore_unrecognized_cops ⇒ Object
Returns the value of attribute ignore_unrecognized_cops.
-
.loaded_features ⇒ Object
readonly
Returns the value of attribute loaded_features.
-
.loaded_plugins ⇒ Object
readonly
Returns the value of attribute loaded_plugins.
Class Method Summary collapse
- .add_excludes_from_files(config, config_file) ⇒ Object
-
.add_loaded_features(loaded_features) ⇒ Object
private
Used to add features that were required inside a config or from the CLI using ‘–require`.
-
.add_loaded_plugins(loaded_plugins) ⇒ Object
private
Used to add plugins that were required inside a config or from the CLI using ‘–plugin`.
- .add_missing_namespaces(path, hash) ⇒ Object
-
.apply_default_overrides(config) ⇒ Object
Applies CLI overrides for ‘AllCops/EnabledByDefault` and `AllCops/DisabledByDefault` to the given configuration.
- .clear_options ⇒ Object
-
.configuration_file_for(target_dir) ⇒ Object
Returns the path of .rubocop.yml searching upwards in the directory structure starting at the given directory where the inspected file is.
- .configuration_from_file(config_file, check: true) ⇒ Object
-
.inject_defaults!(config_yml_path) ⇒ Object
This API is primarily intended for testing and documenting plugins.
-
.load_file(file, check: true) ⇒ Object
rubocop:disable Metrics/AbcSize.
-
.load_yaml_configuration(absolute_path) ⇒ Object
rubocop:enable Metrics/AbcSize.
-
.merge(base_hash, derived_hash) ⇒ Object
Return a recursive merge of two hashes.
-
.merge_with_default(config, config_file, unset_nil: true) ⇒ Object
Merges the given configuration with the default one.
-
.project_root ⇒ Object
deprecated
Deprecated.
Use ‘RuboCop::ConfigFinder.project_root` instead.
Methods included from FileFinder
find_file_upwards, find_last_file_upwards, traverse_directories_upwards
Class Attribute Details
.cache_root(cache_root_override = nil) ⇒ Object
34 35 36 |
# File 'lib/rubocop/config_loader.rb', line 34 def cache_root(cache_root_override = nil) @cache_root ||= CacheConfig.root_dir_from_toplevel_config(cache_root_override) end |
.debug ⇒ Object Also known as: debug?
Returns the value of attribute debug.
26 27 28 |
# File 'lib/rubocop/config_loader.rb', line 26 def debug @debug end |
.default_configuration ⇒ Object
150 151 152 153 154 155 |
# File 'lib/rubocop/config_loader.rb', line 150 def default_configuration @default_configuration ||= begin print 'Default ' if debug? load_file(DEFAULT_FILE) end end |
.disable_pending_cops ⇒ Object
Returns the value of attribute disable_pending_cops.
26 27 28 |
# File 'lib/rubocop/config_loader.rb', line 26 def disable_pending_cops @disable_pending_cops end |
.disabled_by_default ⇒ Object
Returns the value of attribute disabled_by_default.
26 27 28 |
# File 'lib/rubocop/config_loader.rb', line 26 def disabled_by_default @disabled_by_default end |
.enable_pending_cops ⇒ Object
Returns the value of attribute enable_pending_cops.
26 27 28 |
# File 'lib/rubocop/config_loader.rb', line 26 def enable_pending_cops @enable_pending_cops end |
.enabled_by_default ⇒ Object
Returns the value of attribute enabled_by_default.
26 27 28 |
# File 'lib/rubocop/config_loader.rb', line 26 def enabled_by_default @enabled_by_default end |
.ignore_parent_exclusion ⇒ Object Also known as: ignore_parent_exclusion?
Returns the value of attribute ignore_parent_exclusion.
26 27 28 |
# File 'lib/rubocop/config_loader.rb', line 26 def ignore_parent_exclusion @ignore_parent_exclusion end |
.ignore_unrecognized_cops ⇒ Object
Returns the value of attribute ignore_unrecognized_cops.
26 27 28 |
# File 'lib/rubocop/config_loader.rb', line 26 def ignore_unrecognized_cops @ignore_unrecognized_cops end |
.loaded_features ⇒ Object (readonly)
Returns the value of attribute loaded_features.
29 30 31 |
# File 'lib/rubocop/config_loader.rb', line 29 def loaded_features @loaded_features end |
.loaded_plugins ⇒ Object (readonly)
Returns the value of attribute loaded_plugins.
29 30 31 |
# File 'lib/rubocop/config_loader.rb', line 29 def loaded_plugins @loaded_plugins end |
Class Method Details
.add_excludes_from_files(config, config_file) ⇒ Object
140 141 142 143 144 145 146 147 148 |
# File 'lib/rubocop/config_loader.rb', line 140 def add_excludes_from_files(config, config_file) exclusion_file = find_last_file_upwards(DOTFILE, config_file, ConfigFinder.project_root) return unless exclusion_file return if PathUtil.relative_path(exclusion_file) == PathUtil.relative_path(config_file) print 'AllCops/Exclude ' if debug? config.add_excludes_from_higher_level(load_file(exclusion_file)) end |
.add_loaded_features(loaded_features) ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Used to add features that were required inside a config or from the CLI using ‘–require`.
218 219 220 |
# File 'lib/rubocop/config_loader.rb', line 218 def add_loaded_features(loaded_features) @loaded_features.merge(Array(loaded_features)) end |
.add_loaded_plugins(loaded_plugins) ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Used to add plugins that were required inside a config or from the CLI using ‘–plugin`.
211 212 213 |
# File 'lib/rubocop/config_loader.rb', line 211 def add_loaded_plugins(loaded_plugins) @loaded_plugins.merge(Array(loaded_plugins)) end |
.add_missing_namespaces(path, hash) ⇒ Object
93 94 95 96 97 98 99 100 101 102 103 104 105 106 107 |
# File 'lib/rubocop/config_loader.rb', line 93 def add_missing_namespaces(path, hash) # Using `hash.each_key` will cause the # `can't add a new key into hash during iteration` error obsoletion = ConfigObsoletion.new(hash) hash_keys = hash.keys hash_keys.each do |key| next if obsoletion.deprecated_cop_name?(key) q = Cop::Registry.qualified_cop_name(key, path) next if q == key hash[q] = hash.delete(key) end end |
.apply_default_overrides(config) ⇒ Object
Applies CLI overrides for ‘AllCops/EnabledByDefault` and `AllCops/DisabledByDefault` to the given configuration. Used when the configuration would otherwise be returned without going through `merge_with_default` (e.g. there is no user-supplied `.rubocop.yml`).
199 200 201 202 203 204 205 206 |
# File 'lib/rubocop/config_loader.rb', line 199 def apply_default_overrides(config) return config if @enabled_by_default.nil? && @disabled_by_default.nil? hash = config.transform_values do |params| params.is_a?(Hash) ? params.merge('Enabled' => !@disabled_by_default) : params end Config.new(hash, config.loaded_path) end |
.clear_options ⇒ Object
38 39 40 41 42 43 44 45 46 47 48 49 50 |
# File 'lib/rubocop/config_loader.rb', line 38 def @debug = nil @loaded_plugins = Set.new @loaded_features = Set.new @disable_pending_cops = nil @enable_pending_cops = nil @enabled_by_default = nil @disabled_by_default = nil @ignore_parent_exclusion = nil @ignore_unrecognized_cops = nil @cache_root = nil FileFinder.root_level = nil end |
.configuration_file_for(target_dir) ⇒ Object
Returns the path of .rubocop.yml searching upwards in the directory structure starting at the given directory where the inspected file is. If no .rubocop.yml is found there, the user’s home directory is checked. If there’s no .rubocop.yml there either, the path to the default file is returned.
121 122 123 |
# File 'lib/rubocop/config_loader.rb', line 121 def configuration_file_for(target_dir) ConfigFinder.find_config_path(target_dir) end |
.configuration_from_file(config_file, check: true) ⇒ Object
125 126 127 128 129 130 131 132 133 134 135 136 137 138 |
# File 'lib/rubocop/config_loader.rb', line 125 def configuration_from_file(config_file, check: true) return apply_default_overrides(default_configuration) if config_file == DEFAULT_FILE config = load_file(config_file, check: check) config.validate_after_resolution if check if ignore_parent_exclusion? print 'Ignoring AllCops/Exclude from parent folders' if debug? else add_excludes_from_files(config, config_file) end merge_with_default(config, config_file) end |
.inject_defaults!(config_yml_path) ⇒ Object
This API is primarily intended for testing and documenting plugins. When testing a plugin using ‘rubocop/rspec/support`, the plugin is loaded automatically, so this API is usually not needed. It is intended to be used only when implementing tests that do not use `rubocop/rspec/support`.
161 162 163 164 165 166 167 168 169 170 171 172 173 174 175 176 |
# File 'lib/rubocop/config_loader.rb', line 161 def inject_defaults!(config_yml_path) if Pathname(config_yml_path).directory? warn Rainbow(<<~MESSAGE).yellow, uplevel: 1 Use config YAML file path instead of project root directory. e.g., `path/to/config/default.yml` MESSAGE raise ArgumentError, 'Passing a project root directory to `inject_defaults!` is no longer supported.' end path = config_yml_path.to_s hash = ConfigLoader.load_yaml_configuration(path) config = Config.new(hash, path).tap(&:make_excludes_absolute) @default_configuration = ConfigLoader.merge_with_default(config, path) end |
.load_file(file, check: true) ⇒ Object
rubocop:disable Metrics/AbcSize
53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 |
# File 'lib/rubocop/config_loader.rb', line 53 def load_file(file, check: true) path = file_path(file) hash = load_yaml_configuration(path) rubocop_config = Config.create(hash, path, check: false) plugins = hash.delete('plugins') loaded_plugins = resolver.resolve_plugins(rubocop_config, plugins) add_loaded_plugins(loaded_plugins) loaded_features = resolver.resolve_requires(path, hash) add_loaded_features(loaded_features) resolver.resolve_inheritance_from_gems(hash) resolver.resolve_inheritance(path, hash, file, debug?) hash.delete('inherit_from') # Adding missing namespaces only after resolving requires & inheritance, # since both can introduce new cops that need to be considered here. add_missing_namespaces(path, hash) Config.create(hash, path, check: check) end |
.load_yaml_configuration(absolute_path) ⇒ Object
rubocop:enable Metrics/AbcSize
78 79 80 81 82 83 84 85 86 87 88 89 90 91 |
# File 'lib/rubocop/config_loader.rb', line 78 def load_yaml_configuration(absolute_path) file_contents = read_file(absolute_path) yaml_code = Dir.chdir(File.dirname(absolute_path)) { ERB.new(file_contents).result } yaml_tree = check_duplication(yaml_code, absolute_path) hash = yaml_tree_to_hash(yaml_tree) || {} puts "configuration from #{absolute_path}" if debug? unless hash.is_a?(Hash) raise(ValidationError, "Malformed configuration in #{absolute_path}") end hash end |
.merge(base_hash, derived_hash) ⇒ Object
Return a recursive merge of two hashes. That is, a normal hash merge, with the addition that any value that is a hash, and occurs in both arguments, will also be merged. And so on.
112 113 114 |
# File 'lib/rubocop/config_loader.rb', line 112 def merge(base_hash, derived_hash) resolver.merge(base_hash, derived_hash) end |
.merge_with_default(config, config_file, unset_nil: true) ⇒ Object
Merges the given configuration with the default one.
191 192 193 |
# File 'lib/rubocop/config_loader.rb', line 191 def merge_with_default(config, config_file, unset_nil: true) resolver.merge_with_default(config, config_file, unset_nil: unset_nil) end |
.project_root ⇒ Object
Use ‘RuboCop::ConfigFinder.project_root` instead.
Returns the path RuboCop inferred as the root of the project. No file searches will go past this directory.
181 182 183 184 185 186 187 188 |
# File 'lib/rubocop/config_loader.rb', line 181 def project_root warn Rainbow(<<~WARNING).yellow, uplevel: 1 `RuboCop::ConfigLoader.project_root` is deprecated and will be removed in RuboCop 2.0. \ Use `RuboCop::ConfigFinder.project_root` instead. WARNING ConfigFinder.project_root end |