Class: RuboCop::ConfigLoader
- Inherits:
-
Object
- Object
- RuboCop::ConfigLoader
- Extended by:
- FileFinder
- Defined in:
- lib/rubocop/config_loader.rb
Overview
This class represents the configuration of the RuboCop application and all its cops. A Config is associated with a YAML configuration file from which it was read. Several different Configs can be used during a run of the rubocop program, if files in several directories are inspected.
Constant Summary collapse
- DOTFILE =
ConfigFinder::DOTFILE
- RUBOCOP_HOME =
File.realpath(File.join(File.dirname(__FILE__), '..', '..'))
- DEFAULT_FILE =
File.join(RUBOCOP_HOME, 'config', 'default.yml')
- PENDING_BANNER =
"The following cops were added to RuboCop, but are not configured. Please set Enabled to either `true` or `false` in your `.rubocop.yml` file.\n\nPlease also note that you can opt-in to new cops by default by adding this to your config:\n AllCops:\n NewCops: enable\n"
Class Attribute Summary collapse
-
.debug ⇒ Object
(also: debug?)
Returns the value of attribute debug.
- .default_configuration ⇒ Object
-
.disable_pending_cops ⇒ Object
Returns the value of attribute disable_pending_cops.
-
.enable_pending_cops ⇒ Object
Returns the value of attribute enable_pending_cops.
-
.ignore_parent_exclusion ⇒ Object
(also: ignore_parent_exclusion?)
Returns the value of attribute ignore_parent_exclusion.
-
.ignore_unrecognized_cops ⇒ Object
Returns the value of attribute ignore_unrecognized_cops.
-
.loaded_features ⇒ Object
readonly
Returns the value of attribute loaded_features.
Class Method Summary collapse
- .add_excludes_from_files(config, config_file) ⇒ Object
-
.add_loaded_features(loaded_features) ⇒ Object
private
Used to add features that were required inside a config or from the CLI using
--require. - .add_missing_namespaces(path, hash) ⇒ Object
- .clear_options ⇒ Object
-
.configuration_file_for(target_dir) ⇒ Object
Returns the path of .rubocop.yml searching upwards in the directory structure starting at the given directory where the inspected file is.
- .configuration_from_file(config_file, check: true) ⇒ Object
- .inject_defaults!(project_root) ⇒ Object private
- .load_file(file, check: true) ⇒ Object
- .load_yaml_configuration(absolute_path) ⇒ Object
-
.merge(base_hash, derived_hash) ⇒ Object
Return a recursive merge of two hashes.
-
.merge_with_default(config, config_file, unset_nil: true) ⇒ Object
Merges the given configuration with the default one.
- .possible_new_cops?(config) ⇒ Boolean
-
.project_root ⇒ Object
deprecated
Deprecated.
Use
RuboCop::ConfigFinder.project_rootinstead. - .warn_on_pending_cops(pending_cops) ⇒ Object
- .warn_pending_cop(cop) ⇒ Object
Methods included from FileFinder
find_file_upwards, find_last_file_upwards, root_level=, root_level?
Class Attribute Details
.debug ⇒ Object Also known as: debug?
Returns the value of attribute debug.
26 27 28 |
# File 'lib/rubocop/config_loader.rb', line 26 def debug @debug end |
.default_configuration ⇒ Object
133 134 135 136 137 138 |
# File 'lib/rubocop/config_loader.rb', line 133 def default_configuration @default_configuration ||= begin print 'Default ' if debug? load_file(DEFAULT_FILE) end end |
.disable_pending_cops ⇒ Object
Returns the value of attribute disable_pending_cops.
26 27 28 |
# File 'lib/rubocop/config_loader.rb', line 26 def disable_pending_cops @disable_pending_cops end |
.enable_pending_cops ⇒ Object
Returns the value of attribute enable_pending_cops.
26 27 28 |
# File 'lib/rubocop/config_loader.rb', line 26 def enable_pending_cops @enable_pending_cops end |
.ignore_parent_exclusion ⇒ Object Also known as: ignore_parent_exclusion?
Returns the value of attribute ignore_parent_exclusion.
26 27 28 |
# File 'lib/rubocop/config_loader.rb', line 26 def ignore_parent_exclusion @ignore_parent_exclusion end |
.ignore_unrecognized_cops ⇒ Object
Returns the value of attribute ignore_unrecognized_cops.
26 27 28 |
# File 'lib/rubocop/config_loader.rb', line 26 def ignore_unrecognized_cops @ignore_unrecognized_cops end |
.loaded_features ⇒ Object (readonly)
Returns the value of attribute loaded_features.
29 30 31 |
# File 'lib/rubocop/config_loader.rb', line 29 def loaded_features @loaded_features end |
Class Method Details
.add_excludes_from_files(config, config_file) ⇒ Object
123 124 125 126 127 128 129 130 131 |
# File 'lib/rubocop/config_loader.rb', line 123 def add_excludes_from_files(config, config_file) exclusion_file = find_last_file_upwards(DOTFILE, config_file, ConfigFinder.project_root) return unless exclusion_file return if PathUtil.relative_path(exclusion_file) == PathUtil.relative_path(config_file) print 'AllCops/Exclude ' if debug? config.add_excludes_from_higher_level(load_file(exclusion_file)) end |
.add_loaded_features(loaded_features) ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Used to add features that were required inside a config or from the CLI using --require.
194 195 196 |
# File 'lib/rubocop/config_loader.rb', line 194 def add_loaded_features(loaded_features) @loaded_features.merge(Array(loaded_features)) end |
.add_missing_namespaces(path, hash) ⇒ Object
73 74 75 76 77 78 79 80 81 82 83 |
# File 'lib/rubocop/config_loader.rb', line 73 def add_missing_namespaces(path, hash) # Using `hash.each_key` will cause the # `can't add a new key into hash during iteration` error hash_keys = hash.keys hash_keys.each do |key| q = Cop::Registry.qualified_cop_name(key, path) next if q == key hash[q] = hash.delete(key) end end |
.clear_options ⇒ Object
34 35 36 37 38 |
# File 'lib/rubocop/config_loader.rb', line 34 def @debug = nil @loaded_features = Set.new FileFinder.root_level = nil end |
.configuration_file_for(target_dir) ⇒ Object
Returns the path of .rubocop.yml searching upwards in the directory structure starting at the given directory where the inspected file is. If no .rubocop.yml is found there, the user’s home directory is checked. If there’s no .rubocop.yml there either, the path to the default file is returned.
97 98 99 |
# File 'lib/rubocop/config_loader.rb', line 97 def configuration_file_for(target_dir) ConfigFinder.find_config_path(target_dir) end |
.configuration_from_file(config_file, check: true) ⇒ Object
101 102 103 104 105 106 107 108 109 110 111 112 113 114 115 116 |
# File 'lib/rubocop/config_loader.rb', line 101 def configuration_from_file(config_file, check: true) return default_configuration if config_file == DEFAULT_FILE config = load_file(config_file, check: check) config.validate_after_resolution if check if ignore_parent_exclusion? print 'Ignoring AllCops/Exclude from parent folders' if debug? else add_excludes_from_files(config, config_file) end merge_with_default(config, config_file).tap do |merged_config| warn_on_pending_cops(merged_config.pending_cops) unless possible_new_cops?(merged_config) end end |
.inject_defaults!(project_root) ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
141 142 143 144 145 146 147 |
# File 'lib/rubocop/config_loader.rb', line 141 def inject_defaults!(project_root) path = File.join(project_root, 'config', 'default.yml') config = load_file(path) new_config = ConfigLoader.merge_with_default(config, path) puts "configuration from #{path}" if debug? @default_configuration = new_config end |
.load_file(file, check: true) ⇒ Object
40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 |
# File 'lib/rubocop/config_loader.rb', line 40 def load_file(file, check: true) path = file_path(file) hash = load_yaml_configuration(path) loaded_features = resolver.resolve_requires(path, hash) add_loaded_features(loaded_features) resolver.override_department_setting_for_cops({}, hash) resolver.resolve_inheritance_from_gems(hash) resolver.resolve_inheritance(path, hash, file, debug?) hash.delete('inherit_from') # Adding missing namespaces only after resolving requires & inheritance, # since both can introduce new cops that need to be considered here. add_missing_namespaces(path, hash) Config.create(hash, path, check: check) end |
.load_yaml_configuration(absolute_path) ⇒ Object
60 61 62 63 64 65 66 67 68 69 70 71 |
# File 'lib/rubocop/config_loader.rb', line 60 def load_yaml_configuration(absolute_path) file_contents = read_file(absolute_path) yaml_code = Dir.chdir(File.dirname(absolute_path)) { ERB.new(file_contents).result } check_duplication(yaml_code, absolute_path) hash = yaml_safe_load(yaml_code, absolute_path) || {} puts "configuration from #{absolute_path}" if debug? raise(TypeError, "Malformed configuration in #{absolute_path}") unless hash.is_a?(Hash) hash end |
.merge(base_hash, derived_hash) ⇒ Object
Return a recursive merge of two hashes. That is, a normal hash merge, with the addition that any value that is a hash, and occurs in both arguments, will also be merged. And so on.
88 89 90 |
# File 'lib/rubocop/config_loader.rb', line 88 def merge(base_hash, derived_hash) resolver.merge(base_hash, derived_hash) end |
.merge_with_default(config, config_file, unset_nil: true) ⇒ Object
Merges the given configuration with the default one.
187 188 189 |
# File 'lib/rubocop/config_loader.rb', line 187 def merge_with_default(config, config_file, unset_nil: true) resolver.merge_with_default(config, config_file, unset_nil: unset_nil) end |
.possible_new_cops?(config) ⇒ Boolean
118 119 120 121 |
# File 'lib/rubocop/config_loader.rb', line 118 def possible_new_cops?(config) disable_pending_cops || enable_pending_cops || config.disabled_new_cops? || config.enabled_new_cops? end |
.project_root ⇒ Object
Use RuboCop::ConfigFinder.project_root instead.
Returns the path RuboCop inferred as the root of the project. No file searches will go past this directory.
152 153 154 155 156 157 158 159 |
# File 'lib/rubocop/config_loader.rb', line 152 def project_root warn Rainbow(" `RuboCop::ConfigLoader.project_root` is deprecated and will be removed in RuboCop 2.0. \\\n Use `RuboCop::ConfigFinder.project_root` instead.\n WARNING\n\n ConfigFinder.project_root\nend\n").yellow |
.warn_on_pending_cops(pending_cops) ⇒ Object
169 170 171 172 173 174 175 176 177 |
# File 'lib/rubocop/config_loader.rb', line 169 def warn_on_pending_cops(pending_cops) return if pending_cops.empty? warn Rainbow(PENDING_BANNER).yellow pending_cops.each { |cop| warn_pending_cop cop } warn Rainbow('For more information: https://docs.rubocop.org/rubocop/versioning.html').yellow end |
.warn_pending_cop(cop) ⇒ Object
179 180 181 182 183 184 |
# File 'lib/rubocop/config_loader.rb', line 179 def warn_pending_cop(cop) version = cop.['VersionAdded'] || 'N/A' warn Rainbow("#{cop.name}: # new in #{version}").yellow warn Rainbow(' Enabled: true').yellow end |