Class: RuboCop::Cop::Performance::Casecmp
- Defined in:
- lib/rubocop/cop/performance/casecmp.rb
Overview
This cop identifies places where a case-insensitive string comparison
can better be implemented using casecmp.
Constant Summary collapse
- MSG =
'Use `casecmp` instead of `%s %s`.'.freeze
Constants included from Util
Util::ASGN_NODES, Util::EQUALS_ASGN_NODES, Util::LITERAL_REGEX, Util::OPERATOR_METHODS, Util::SHORTHAND_ASGN_NODES, Util::STRING_ESCAPES, Util::STRING_ESCAPE_REGEX
Instance Attribute Summary
Attributes inherited from Cop
#config, #corrections, #offenses, #processed_source
Instance Method Summary collapse
Methods inherited from Cop
#add_offense, all, #config_to_allow_offenses, #config_to_allow_offenses=, #cop_config, cop_name, #cop_name, cop_type, #correct, #debug?, #details, #display_cop_names?, #display_style_guide?, #excluded_file?, #extra_details?, inherited, #initialize, #join_force?, lint?, match?, #message, non_rails, #parse, qualified_cop_name, #reference_url, #relevant_file?, #style_guide_url, #target_ruby_version
Methods included from Sexp
Methods included from NodePattern::Macros
#def_node_matcher, #def_node_search, #node_search_body
Methods included from AutocorrectLogic
#autocorrect?, #autocorrect_enabled?, #autocorrect_requested?, #support_autocorrect?
Methods included from IgnoredNode
#ignore_node, #ignored_node?, #part_of_ignored_node?
Methods included from Util
begins_its_line?, block_length, comment_line?, directions, double_quotes_acceptable?, double_quotes_required?, ends_its_line?, first_part_of_call_chain, interpret_string_escapes, line_range, move_pos, numeric_range_size, on_node, operator?, parentheses?, range_with_surrounding_comma, range_with_surrounding_space, source_range, strip_quotes, to_string_literal, to_symbol_literal, within_node?
Methods included from PathUtil
hidden?, issue_deprecation_warning, match_path?, relative_path
Constructor Details
This class inherits a constructor from RuboCop::Cop::Cop
Instance Method Details
#autocorrect(node) ⇒ Object
31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 |
# File 'lib/rubocop/cop/performance/casecmp.rb', line 31 def autocorrect(node) receiver, _method, arg = *node range = Parser::Source::Range.new(node.source_range.source_buffer, receiver.loc.selector.begin_pos, arg.loc.begin.begin_pos) lambda do |corrector| # we want resulting call to be parenthesized # if arg already includes one or more sets of parens, don't add more # or if method call already used parens, again, don't add more if arg.loc.begin.source == '(' corrector.replace(range, 'casecmp') elsif range.source =~ /\(/ corrector.replace(range, 'casecmp(') else corrector.replace(range, 'casecmp(') corrector.insert_after(arg.source_range, ')') end end end |
#on_send(node) ⇒ Object
24 25 26 27 28 29 |
# File 'lib/rubocop/cop/performance/casecmp.rb', line 24 def on_send(node) downcase_eq(node) do |send_downcase, case_method, eq_method| range = node.loc.selector.join(send_downcase.loc.selector) add_offense(node, range, format(MSG, case_method, eq_method)) end end |