Class: RuboCop::Cop::Performance::Count
- Defined in:
- lib/rubocop/cop/performance/count.rb
Overview
This cop is used to identify usages of count on an Enumerable that follow calls to select or reject. Querying logic can instead be passed to the count call.
ActiveRecord compatibility: ActiveRecord will ignore the block that is passed to count. Other methods, such as select, will convert the association to an array and then run the block on the array. A simple work around to make count work with a block is to call ‘to_a.count …`.
Example:
Model.where(id: [1, 2, 3].select { |m| m.method == true }.size
becomes:
Model.where(id: [1, 2, 3]).to_a.count { |m| m.method == true }
Constant Summary collapse
- MSG =
'Use `count` instead of `%s...%s`.'.freeze
- SELECTORS =
[:reject, :select].freeze
- COUNTERS =
[:count, :length, :size].freeze
Constants included from Util
Util::ASGN_NODES, Util::BYTE_ORDER_MARK, Util::EQUALS_ASGN_NODES, Util::LITERAL_REGEX, Util::OPERATOR_METHODS, Util::SHORTHAND_ASGN_NODES
Instance Attribute Summary
Attributes inherited from Cop
#config, #corrections, #offenses, #processed_source
Instance Method Summary collapse
Methods inherited from Cop
#add_offense, all, #config_to_allow_offenses, #config_to_allow_offenses=, #cop_config, cop_name, #cop_name, cop_type, #correct, #debug?, #details, #display_cop_names?, #display_style_guide?, #excluded_file?, #extra_details?, #highlights, inherited, #initialize, #join_force?, lint?, match?, #message, #messages, non_rails, qualified_cop_name, #reference_url, #relevant_file?, #style_guide_url, #target_ruby_version
Methods included from Sexp
Methods included from NodePattern::Macros
#def_node_matcher, #def_node_search, #node_search_body
Methods included from AutocorrectLogic
#autocorrect?, #autocorrect_enabled?, #autocorrect_requested?, #support_autocorrect?
Methods included from IgnoredNode
#ignore_node, #ignored_node?, #part_of_ignored_node?
Methods included from Util
begins_its_line?, block_length, comment_line?, directions, double_quotes_acceptable?, double_quotes_required?, effective_column, ends_its_line?, escape_string, first_part_of_call_chain, interpret_string_escapes, line_range, move_pos, needs_escaping?, numeric_range_size, on_node, operator?, parentheses?, parenthesized_call?, range_with_surrounding_comma, range_with_surrounding_space, source_range, strip_quotes, to_string_literal, to_symbol_literal, within_node?
Methods included from PathUtil
absolute?, match_path?, relative_path
Constructor Details
This class inherits a constructor from RuboCop::Cop::Cop
Instance Method Details
#autocorrect(node) ⇒ Object
62 63 64 65 66 67 68 69 70 71 72 73 74 75 |
# File 'lib/rubocop/cop/performance/count.rb', line 62 def autocorrect(node) selector, selector_loc = parse(node) return if selector == :reject range = Parser::Source::Range.new(node.source_range.source_buffer, node.loc.dot.begin_pos, node.source_range.end_pos) lambda do |corrector| corrector.remove(range) corrector.replace(selector_loc, 'count') end end |
#on_send(node) ⇒ Object
47 48 49 50 51 52 53 54 55 56 57 58 59 60 |
# File 'lib/rubocop/cop/performance/count.rb', line 47 def on_send(node) return unless should_run? selector, selector_loc, params, counter = parse(node) return unless COUNTERS.include?(counter) return unless SELECTORS.include?(selector) return if params && !params.block_pass_type? return if node.parent && node.parent.block_type? range = Parser::Source::Range.new(node.source_range.source_buffer, selector_loc.begin_pos, node.source_range.end_pos) add_offense(node, range, format(MSG, selector, counter)) end |