Class: RuboCop::Cop::Style::AccessorGrouping
- Extended by:
- AutoCorrector
- Includes:
- ConfigurableEnforcedStyle, RangeHelp, VisibilityHelp
- Defined in:
- lib/rubocop/cop/style/accessor_grouping.rb
Overview
Checks for grouping of accessors in class and module bodies.
By default it enforces accessors to be placed in grouped declarations,
but it can be configured to enforce separating them in multiple declarations.
NOTE: Sorbet is not compatible with "grouped" style. Consider "separated" style
or disabling this cop.
Constant Summary collapse
- GROUPED_MSG =
'Group together all `%<accessor>s` attributes.'- SEPARATED_MSG =
'Use one attribute per `%<accessor>s`.'- ACCESSOR_METHODS =
i[attr_reader attr_writer attr_accessor attr].freeze
Constants included from VisibilityHelp
VisibilityHelp::VISIBILITY_SCOPES
Constants inherited from Base
Instance Attribute Summary
Attributes inherited from Base
Instance Method Summary collapse
- #on_class(node) ⇒ Object (also: #on_sclass, #on_module)
Methods included from AutoCorrector
Methods included from ConfigurableEnforcedStyle
#alternative_style, #alternative_styles, #ambiguous_style_detected, #correct_style_detected, #detected_style, #detected_style=, #no_acceptable_style!, #no_acceptable_style?, #opposite_style_detected, #style, #style_configured?, #style_detected, #style_parameter_name, #supported_styles, #unexpected_style_detected
Methods inherited from Base
#active_support_extensions_enabled?, #add_global_offense, #add_offense, autocorrect_incompatible_with, badge, callbacks_needed, #callbacks_needed, #config_to_allow_offenses, #config_to_allow_offenses=, #cop_config, cop_name, #cop_name, department, documentation_url, exclude_from_registry, #excluded_file?, #external_dependency_checksum, inherited, #initialize, joining_forces, lint?, match?, #offenses, #on_investigation_end, #on_new_investigation, #on_other_file, #parse, #ready, #relevant_file?, support_autocorrect?, support_multiple_source?, #target_rails_version, #target_ruby_version
Methods included from ExcludeLimit
Methods included from AutocorrectLogic
#autocorrect?, #autocorrect_enabled?, #autocorrect_requested?, #autocorrect_with_disable_uncorrectable?, #correctable?, #disable_uncorrectable?, #safe_autocorrect?
Methods included from IgnoredNode
#ignore_node, #ignored_node?, #part_of_ignored_node?
Methods included from Util
Constructor Details
This class inherits a constructor from RuboCop::Cop::Base
Instance Method Details
#on_class(node) ⇒ Object Also known as: on_sclass, on_module
48 49 50 51 52 53 54 |
# File 'lib/rubocop/cop/style/accessor_grouping.rb', line 48 def on_class(node) class_send_elements(node).each do |macro| next unless accessor?(macro) check(macro) end end |