Class: RuboCop::Cop::Style::ConditionalAssignment
- Extended by:
- AutoCorrector
- Defined in:
- lib/rubocop/cop/style/conditional_assignment.rb
Overview
Checks for if and case statements where each branch is used for
both the assignment and comparison of the same variable
when using the return of the condition can be used instead.
Constant Summary collapse
- MSG =
'Use the return of the conditional for variable assignment and comparison.'- ASSIGN_TO_CONDITION_MSG =
'Assign variables inside of conditionals.'- VARIABLE_ASSIGNMENT_TYPES =
%i[casgn cvasgn gvasgn ivasgn lvasgn].freeze
- ASSIGNMENT_TYPES =
VARIABLE_ASSIGNMENT_TYPES + %i[and_asgn or_asgn op_asgn masgn].freeze
- ENABLED =
'Enabled'- SINGLE_LINE_CONDITIONS_ONLY =
'SingleLineConditionsOnly'
Constants included from ConditionalAssignmentHelper
RuboCop::Cop::Style::ConditionalAssignmentHelper::ALIGN_WITH, RuboCop::Cop::Style::ConditionalAssignmentHelper::END_ALIGNMENT, RuboCop::Cop::Style::ConditionalAssignmentHelper::EQUAL, RuboCop::Cop::Style::ConditionalAssignmentHelper::KEYWORD
Constants inherited from Base
Instance Attribute Summary
Attributes inherited from Base
#config, #processed_source, #project_index
Instance Method Summary collapse
-
#assignment_type?(node) ⇒ Object
The shovel operator
<<does not have its own type. - #on_case(node) ⇒ Object
- #on_case_match(node) ⇒ Object
- #on_if(node) ⇒ Object
- #on_send(node) ⇒ Object
Methods included from AutoCorrector
Methods included from ConfigurableEnforcedStyle
#alternative_style, #alternative_styles, #ambiguous_style_detected, #correct_style_detected, #detected_style, #detected_style=, #no_acceptable_style!, #no_acceptable_style?, #opposite_style_detected, #style, #style_configured?, #style_detected, #style_parameter_name, #supported_styles, #unexpected_style_detected
Methods included from ConditionalAssignmentHelper
#end_with_eq?, #expand_elses, #expand_when_branches, #indent, #lhs, #tail
Methods inherited from Base
#active_support_extensions_enabled?, #add_global_offense, #add_offense, #always_autocorrect?, autocorrect_incompatible_with, badge, #begin_investigation, #callbacks_needed, callbacks_needed, #config_to_allow_offenses, #config_to_allow_offenses=, #contextual_autocorrect?, #cop_config, cop_name, #cop_name, department, documentation_url, exclude_from_registry, #excluded_file?, #external_dependency_checksum, inherited, #initialize, #inspect, joining_forces, lint?, match?, #message, #offenses, #on_investigation_end, #on_new_investigation, #on_other_file, #parse, #parser_engine, #ready, #relevant_file?, requires_gem, #string_literals_frozen_by_default?, support_autocorrect?, support_multiple_source?, #target_gem_version, #target_rails_version, #target_ruby_version
Methods included from ExcludeLimit
cop_dir_for, #exclude_limit, read_limits
Methods included from AutocorrectLogic
#autocorrect?, #autocorrect_enabled?, #autocorrect_requested?, #autocorrect_with_disable_uncorrectable?, #correctable?, #disable_uncorrectable?, #safe_autocorrect?
Methods included from IgnoredNode
#ignore_node, #ignored_node?, #part_of_ignored_node?
Methods included from Util
Constructor Details
This class inherits a constructor from RuboCop::Cop::Base
Instance Method Details
#assignment_type?(node) ⇒ Object
The shovel operator << does not have its own type. It is a send
type.
218 219 220 221 222 223 |
# File 'lib/rubocop/cop/style/conditional_assignment.rb', line 218 def_node_matcher :assignment_type?, <<~PATTERN { #{ASSIGNMENT_TYPES.join(' ')} (send _recv {:[]= :<< :=~ :!~ :<=> #end_with_eq? :< :>} ...) } PATTERN |
#on_case(node) ⇒ Object
254 255 256 257 258 259 260 261 262 |
# File 'lib/rubocop/cop/style/conditional_assignment.rb', line 254 def on_case(node) return unless style == :assign_to_condition return unless node.else_branch when_branches = (node.when_branches) branches = [*when_branches, node.else_branch] check_node(node, branches) end |
#on_case_match(node) ⇒ Object
264 265 266 267 268 269 270 271 272 |
# File 'lib/rubocop/cop/style/conditional_assignment.rb', line 264 def on_case_match(node) return unless style == :assign_to_condition return unless node.else_branch in_pattern_branches = (node.in_pattern_branches) branches = [*in_pattern_branches, node.else_branch] check_node(node, branches) end |
#on_if(node) ⇒ Object
240 241 242 243 244 245 246 247 248 249 250 251 252 |
# File 'lib/rubocop/cop/style/conditional_assignment.rb', line 240 def on_if(node) return unless style == :assign_to_condition return if node.elsif? else_branch = node.else_branch elsif_branches, else_branch = (else_branch) return unless else_branch branches = [node.if_branch, *elsif_branches, else_branch] check_node(node, branches) end |
#on_send(node) ⇒ Object
234 235 236 237 238 |
# File 'lib/rubocop/cop/style/conditional_assignment.rb', line 234 def on_send(node) return unless assignment_type?(node) check_assignment_to_condition(node) end |