Class: RuboCop::Cop::Style::ConditionalAssignment
- Defined in:
- lib/rubocop/cop/style/conditional_assignment.rb
Overview
Check for if and case statements where each branch is used for assignment to the same variable when using the return of the condition can be used instead.
Constant Summary collapse
- MSG =
'Use the return of the conditional for variable assignment ' \ 'and comparison.'.freeze
- ASSIGN_TO_CONDITION_MSG =
'Assign variables inside of conditionals'.freeze
- VARIABLE_ASSIGNMENT_TYPES =
%i[casgn cvasgn gvasgn ivasgn lvasgn].freeze
- ASSIGNMENT_TYPES =
VARIABLE_ASSIGNMENT_TYPES + %i[and_asgn or_asgn op_asgn masgn].freeze
- LINE_LENGTH =
'Metrics/LineLength'.freeze
- INDENTATION_WIDTH =
'Layout/IndentationWidth'.freeze
- ENABLED =
'Enabled'.freeze
- MAX =
'Max'.freeze
- SINGLE_LINE_CONDITIONS_ONLY =
'SingleLineConditionsOnly'.freeze
- WIDTH =
'Width'.freeze
Constants included from ConditionalAssignmentHelper
RuboCop::Cop::Style::ConditionalAssignmentHelper::ALIGN_WITH, RuboCop::Cop::Style::ConditionalAssignmentHelper::END_ALIGNMENT, RuboCop::Cop::Style::ConditionalAssignmentHelper::EQUAL, RuboCop::Cop::Style::ConditionalAssignmentHelper::KEYWORD
Constants included from Util
Util::ASGN_NODES, Util::BYTE_ORDER_MARK, Util::CONDITIONAL_NODES, Util::EQUALS_ASGN_NODES, Util::LITERAL_REGEX, Util::LOGICAL_OPERATOR_NODES, Util::MODIFIER_NODES, Util::OPERATOR_METHODS, Util::SHORTHAND_ASGN_NODES
Instance Attribute Summary
Attributes inherited from Cop
#config, #corrections, #offenses, #processed_source
Instance Method Summary collapse
Methods included from IgnoredNode
#ignore_node, #ignored_node?, #part_of_ignored_node?
Methods included from ConfigurableEnforcedStyle
#alternative_style, #alternative_styles, #ambiguous_style_detected, #correct_style_detected, #detected_style, #detected_style=, #no_acceptable_style!, #no_acceptable_style?, #opposite_style_detected, #style, #style_detected, #style_parameter_name, #supported_styles, #unexpected_style_detected
Methods included from ConditionalAssignmentHelper
#end_with_eq?, #expand_elses, #expand_when_branches, #indent, #lhs, #tail
Methods inherited from Cop
#add_offense, all, autocorrect_incompatible_with, badge, #config_to_allow_offenses, #config_to_allow_offenses=, #cop_config, cop_name, #cop_name, #correct, department, #duplicate_location?, #excluded_file?, #find_location, #highlights, inherited, #initialize, #join_force?, lint?, match?, #message, #messages, non_rails, #parse, qualified_cop_name, #relevant_file?, #target_rails_version, #target_ruby_version
Methods included from AST::Sexp
Methods included from NodePattern::Macros
#def_node_matcher, #def_node_search, #node_search, #node_search_all, #node_search_body, #node_search_first
Methods included from AutocorrectLogic
#autocorrect?, #autocorrect_enabled?, #autocorrect_requested?, #support_autocorrect?
Methods included from Util
begins_its_line?, block_length, comment_line?, compatible_external_encoding_for?, directions, double_quotes_required?, effective_column, ends_its_line?, escape_string, first_part_of_call_chain, interpret_string_escapes, line_distance, line_range, move_pos, needs_escaping?, on_node, operator?, parentheses?, parenthesized_call?, preceed?, range_between, range_by_whole_lines, range_with_surrounding_comma, range_with_surrounding_space, same_line?, source_range, strip_quotes, stripped_source_upto, symbol_without_quote?, to_string_literal, to_supported_styles, to_symbol_literal, within_node?
Methods included from PathUtil
absolute?, match_path?, relative_path, smart_path
Constructor Details
This class inherits a constructor from RuboCop::Cop::Cop
Instance Method Details
#on_case(node) ⇒ Object
257 258 259 260 261 262 263 264 265 |
# File 'lib/rubocop/cop/style/conditional_assignment.rb', line 257 def on_case(node) return unless style == :assign_to_condition return unless node.else_branch when_branches = (node.when_branches) branches = [*when_branches, node.else_branch] check_node(node, branches) end |
#on_if(node) ⇒ Object
243 244 245 246 247 248 249 250 251 252 253 254 255 |
# File 'lib/rubocop/cop/style/conditional_assignment.rb', line 243 def on_if(node) return unless style == :assign_to_condition return if node.elsif? else_branch = node.else_branch elsif_branches, else_branch = (else_branch) return unless else_branch branches = [node.if_branch, *elsif_branches, else_branch] check_node(node, branches) end |
#on_send(node) ⇒ Object
237 238 239 240 241 |
# File 'lib/rubocop/cop/style/conditional_assignment.rb', line 237 def on_send(node) return unless assignment_type?(node) check_assignment_to_condition(node) end |