Class: RuboCop::Cop::Style::MethodCallWithArgsParentheses
- Extended by:
- AutoCorrector
- Includes:
- AllowedMethods, AllowedPattern, ConfigurableEnforcedStyle, OmitParentheses, RequireParentheses
- Defined in:
- lib/rubocop/cop/style/method_call_with_args_parentheses.rb,
lib/rubocop/cop/style/method_call_with_args_parentheses/omit_parentheses.rb,
lib/rubocop/cop/style/method_call_with_args_parentheses/require_parentheses.rb
Overview
Enforces the presence (default) or absence of parentheses in method calls containing parameters.
In the default style (require_parentheses), macro methods are allowed. Additional methods can be added to the AllowedMethods or AllowedPatterns list. These options are valid only in the default style. Macros can be included by either setting IgnoreMacros to false or adding specific macros to the IncludedMacros list.
Precedence of options is all follows:
-
AllowedMethods -
AllowedPatterns -
IncludedMacros
eg. If a method is listed in both IncludedMacros and AllowedMethods, then the latter takes precedence (that is, the method is allowed).
In the alternative style (omit_parentheses), there are three additional options.
-
AllowParenthesesInChainingisfalseby default. Setting it totrueallows the presence of parentheses in the last call during method chaining. -
AllowParenthesesInMultilineCallisfalseby default. Setting itto `true` allows the presence of parentheses in multi-line method calls. -
AllowParenthesesInCamelCaseMethodisfalseby default. Thisallows the presence of parentheses when calling a method whose name begins with a capital letter and which has no arguments. Setting it to `true` allows the presence of parentheses in such a method call even with arguments.
NOTE: Parentheses are still allowed in cases where omitting them results in ambiguous or syntactically incorrect code. For example, parentheses are required around a method with arguments when inside an endless method definition introduced in Ruby 3.0. Parentheses are also allowed when forwarding arguments with the triple-dot syntax introduced in Ruby 2.7 as omitting them starts an endless range. And Ruby 3.1’s hash omission syntax has a case that requires parentheses because of the following issue: bugs.ruby-lang.org/issues/18396.
Defined Under Namespace
Modules: OmitParentheses, RequireParentheses
Constant Summary
Constants included from OmitParentheses
OmitParentheses::TRAILING_WHITESPACE_REGEX
Constants inherited from Base
Instance Attribute Summary
Attributes inherited from Base
Class Method Summary collapse
Instance Method Summary collapse
- #on_send(node) ⇒ Object (also: #on_csend, #on_super, #on_yield)
Methods included from AutoCorrector
Methods included from ConfigurableEnforcedStyle
#alternative_style, #alternative_styles, #ambiguous_style_detected, #correct_style_detected, #detected_style, #detected_style=, #no_acceptable_style!, #no_acceptable_style?, #opposite_style_detected, #style, #style_configured?, #style_detected, #style_parameter_name, #supported_styles, #unexpected_style_detected
Methods inherited from Base
#active_support_extensions_enabled?, #add_global_offense, #add_offense, badge, callbacks_needed, #callbacks_needed, #config_to_allow_offenses, #config_to_allow_offenses=, #cop_config, cop_name, #cop_name, department, documentation_url, exclude_from_registry, #excluded_file?, #external_dependency_checksum, inherited, #initialize, joining_forces, lint?, match?, #message, #offenses, #on_investigation_end, #on_new_investigation, #on_other_file, #parse, #ready, #relevant_file?, support_autocorrect?, support_multiple_source?, #target_rails_version, #target_ruby_version
Methods included from ExcludeLimit
Methods included from AutocorrectLogic
#autocorrect?, #autocorrect_enabled?, #autocorrect_requested?, #autocorrect_with_disable_uncorrectable?, #correctable?, #disable_uncorrectable?, #safe_autocorrect?
Methods included from IgnoredNode
#ignore_node, #ignored_node?, #part_of_ignored_node?
Methods included from Util
Constructor Details
This class inherits a constructor from RuboCop::Cop::Base
Class Method Details
.autocorrect_incompatible_with ⇒ Object
206 207 208 |
# File 'lib/rubocop/cop/style/method_call_with_args_parentheses.rb', line 206 def self.autocorrect_incompatible_with [Style::NestedParenthesizedCalls, Style::RescueModifier] end |
Instance Method Details
#on_send(node) ⇒ Object Also known as: on_csend, on_super, on_yield
210 211 212 |
# File 'lib/rubocop/cop/style/method_call_with_args_parentheses.rb', line 210 def on_send(node) send(style, node) # call require_parentheses or omit_parentheses end |