Class: RuboCop::Cop::Style::MixinGrouping
- Includes:
- ConfigurableEnforcedStyle
- Defined in:
- lib/rubocop/cop/style/mixin_grouping.rb
Overview
This cop checks for grouping of mixins in class and module bodies. By default it enforces mixins to be placed in separate declarations, but it can be configured to enforce grouping them in one declaration.
Constant Summary collapse
- MIXIN_METHODS =
i[extend include prepend].freeze
- MSG =
'Put `%<mixin>s` mixins in %<suffix>s.'.freeze
Constants included from Util
Util::ASGN_NODES, Util::CONDITIONAL_NODES, Util::EQUALS_ASGN_NODES, Util::LITERAL_REGEX, Util::LOGICAL_OPERATOR_NODES, Util::MODIFIER_NODES, Util::OPERATOR_METHODS, Util::SHORTHAND_ASGN_NODES
Instance Attribute Summary
Attributes inherited from Cop
#config, #corrections, #offenses, #processed_source
Instance Method Summary collapse
- #autocorrect(node) ⇒ Object
- #on_class(node) ⇒ Object (also: #on_module)
Methods included from ConfigurableEnforcedStyle
#alternative_style, #alternative_styles, #ambiguous_style_detected, #correct_style_detected, #detected_style, #detected_style=, #no_acceptable_style!, #no_acceptable_style?, #opposite_style_detected, #style, #style_detected, #style_parameter_name, #supported_styles, #unexpected_style_detected
Methods inherited from Cop
#add_offense, all, autocorrect_incompatible_with, badge, #config_to_allow_offenses, #config_to_allow_offenses=, #cop_config, #cop_name, cop_name, #correct, department, #duplicate_location?, #excluded_file?, #find_location, #highlights, inherited, #initialize, #join_force?, lint?, match?, #messages, non_rails, #parse, qualified_cop_name, #relevant_file?, #target_rails_version, #target_ruby_version
Methods included from AST::Sexp
Methods included from NodePattern::Macros
#def_node_matcher, #def_node_search, #node_search, #node_search_all, #node_search_body, #node_search_first
Methods included from AutocorrectLogic
#autocorrect?, #autocorrect_enabled?, #autocorrect_requested?, #support_autocorrect?
Methods included from IgnoredNode
#ignore_node, #ignored_node?, #part_of_ignored_node?
Methods included from Util
begins_its_line?, comment_line?, double_quotes_required?, escape_string, first_part_of_call_chain, interpret_string_escapes, line_range, needs_escaping?, on_node, operator?, parentheses?, same_line?, to_string_literal, to_supported_styles, tokens
Methods included from PathUtil
absolute?, match_path?, pwd, relative_path, reset_pwd, smart_path
Constructor Details
This class inherits a constructor from RuboCop::Cop::Cop
Instance Method Details
#autocorrect(node) ⇒ Object
50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 |
# File 'lib/rubocop/cop/style/mixin_grouping.rb', line 50 def autocorrect(node) range = node.loc.expression if separated_style? correction = separate_mixins(node) else mixins = sibling_mixins(node) if node == mixins.first correction = group_mixins(node, mixins) else range = range_to_remove_for_subsequent_mixin(mixins, node) correction = '' end end ->(corrector) { corrector.replace(range, correction) } end |
#on_class(node) ⇒ Object Also known as: on_module
39 40 41 42 43 44 45 46 |
# File 'lib/rubocop/cop/style/mixin_grouping.rb', line 39 def on_class(node) begin_node = node.child_nodes.find(&:begin_type?) || node begin_node.each_child_node(:send).select(&:macro?).each do |macro| next unless MIXIN_METHODS.include?(macro.method_name) check(macro) end end |