Class: RuboCop::Cop::Style::ParallelAssignment
- Extended by:
- AutoCorrector
- Includes:
- RescueNode
- Defined in:
- lib/rubocop/cop/style/parallel_assignment.rb
Overview
Checks for simple usages of parallel assignment. Parallel assignment is less readable than individual assignments and makes it harder to follow what each variable is being set to.
This will only complain when the number of variables being assigned matched the number of assigning variables.
Defined Under Namespace
Classes: AssignmentSorter, GenericCorrector, ModifierCorrector, RescueCorrector
Constant Summary collapse
- MSG =
'Do not use parallel assignment.'
Constants inherited from Base
Instance Attribute Summary
Attributes inherited from Base
Instance Method Summary collapse
-
#on_masgn(node) ⇒ Object
rubocop:disable Metrics/AbcSize.
Methods included from AutoCorrector
Methods included from RescueNode
Methods inherited from Base
#active_support_extensions_enabled?, #add_global_offense, #add_offense, #always_autocorrect?, autocorrect_incompatible_with, badge, #begin_investigation, #callbacks_needed, callbacks_needed, #config_to_allow_offenses, #config_to_allow_offenses=, #contextual_autocorrect?, #cop_config, #cop_name, cop_name, department, documentation_url, exclude_from_registry, #excluded_file?, #external_dependency_checksum, inherited, #initialize, #inspect, joining_forces, lint?, match?, #message, #offenses, #on_investigation_end, #on_new_investigation, #on_other_file, #parse, #parser_engine, #ready, #relevant_file?, requires_gem, #string_literals_frozen_by_default?, support_autocorrect?, support_multiple_source?, #target_gem_version, #target_rails_version, #target_ruby_version
Methods included from ExcludeLimit
Methods included from AutocorrectLogic
#autocorrect?, #autocorrect_enabled?, #autocorrect_requested?, #autocorrect_with_disable_uncorrectable?, #correctable?, #disable_uncorrectable?, #safe_autocorrect?
Methods included from IgnoredNode
#ignore_node, #ignored_node?, #part_of_ignored_node?
Methods included from Util
Constructor Details
This class inherits a constructor from RuboCop::Cop::Base
Instance Method Details
#on_masgn(node) ⇒ Object
rubocop:disable Metrics/AbcSize
33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 |
# File 'lib/rubocop/cop/style/parallel_assignment.rb', line 33 def on_masgn(node) # rubocop:disable Metrics/AbcSize return if part_of_ignored_node?(node) rhs = node.rhs rhs = rhs.body if rhs.rescue_type? rhs_elements = Array(rhs).compact # edge case for one constant return if allowed_lhs?(node.assignments) || allowed_rhs?(rhs) || allowed_masign?(node.assignments, rhs_elements) range = node.source_range.begin.join(rhs.source_range.end) add_offense(range) do |corrector| autocorrect(corrector, node, rhs) end ignore_node(node) end |