Class: RuboCop::Cop::Style::TernaryParentheses
- Defined in:
- lib/rubocop/cop/style/ternary_parentheses.rb
Overview
This cop checks for the presence of parentheses around ternary conditions. It is configurable to enforce inclusion or omission of parentheses using EnforcedStyle. Omission is only enforced when removing the parentheses won’t cause a different behavior.
Constant Summary collapse
- VARIABLE_TYPES =
AST::Node::VARIABLES
- NON_COMPLEX_TYPES =
[*VARIABLE_TYPES, :const, :defined?, :yield].freeze
- MSG =
'%<command>s parentheses for ternary conditions.'.freeze
- MSG_COMPLEX =
'%<command>s parentheses for ternary expressions with' \ ' complex conditions.'.freeze
Constants included from RuboCop::Cop::SurroundingSpace
RuboCop::Cop::SurroundingSpace::NO_SPACE_COMMAND, RuboCop::Cop::SurroundingSpace::SPACE_COMMAND
Constants included from Util
Util::ASGN_NODES, Util::CONDITIONAL_NODES, Util::EQUALS_ASGN_NODES, Util::LITERAL_REGEX, Util::LOGICAL_OPERATOR_NODES, Util::MODIFIER_NODES, Util::OPERATOR_METHODS, Util::SHORTHAND_ASGN_NODES
Instance Attribute Summary
Attributes inherited from Cop
#config, #corrections, #offenses, #processed_source
Instance Method Summary collapse
Methods included from ConfigurableEnforcedStyle
#alternative_style, #alternative_styles, #ambiguous_style_detected, #correct_style_detected, #detected_style, #detected_style=, #no_acceptable_style!, #no_acceptable_style?, #opposite_style_detected, #style, #style_detected, #style_parameter_name, #supported_styles, #unexpected_style_detected
Methods included from NodePattern::Macros
#def_node_matcher, #def_node_search, #node_search, #node_search_all, #node_search_body, #node_search_first
Methods inherited from Cop
#add_offense, all, autocorrect_incompatible_with, badge, #config_to_allow_offenses, #config_to_allow_offenses=, #cop_config, #cop_name, cop_name, #correct, department, #duplicate_location?, #excluded_file?, #find_location, #highlights, inherited, #initialize, #join_force?, lint?, match?, #messages, non_rails, #parse, qualified_cop_name, #relevant_file?, #target_rails_version, #target_ruby_version
Methods included from AST::Sexp
Methods included from AutocorrectLogic
#autocorrect?, #autocorrect_enabled?, #autocorrect_requested?, #support_autocorrect?
Methods included from IgnoredNode
#ignore_node, #ignored_node?, #part_of_ignored_node?
Methods included from Util
begins_its_line?, comment_line?, double_quotes_required?, escape_string, first_part_of_call_chain, interpret_string_escapes, line_range, needs_escaping?, on_node, operator?, parentheses?, same_line?, to_string_literal, to_supported_styles, tokens
Methods included from PathUtil
absolute?, match_path?, pwd, relative_path, reset_pwd, smart_path
Constructor Details
This class inherits a constructor from RuboCop::Cop::Cop
Instance Method Details
#autocorrect(node) ⇒ Object
61 62 63 64 65 66 67 68 69 70 71 72 73 |
# File 'lib/rubocop/cop/style/ternary_parentheses.rb', line 61 def autocorrect(node) condition = node.condition return nil if parenthesized?(condition) && (safe_assignment?(condition) || unsafe_autocorrect?(condition)) if parenthesized?(condition) correct_parenthesized(condition) else correct_unparenthesized(condition) end end |
#on_if(node) ⇒ Object
55 56 57 58 59 |
# File 'lib/rubocop/cop/style/ternary_parentheses.rb', line 55 def on_if(node) return unless node.ternary? && !infinite_loop? && offense?(node) add_offense(node, location: node.source_range) end |