Module: RuboCop::Cop::Util
Overview
This module contains a collection of useful utility methods.
Constant Summary collapse
- BYTE_ORDER_MARK =
The Unicode codepoint
0xfeff- EQUALS_ASGN_NODES =
[:lvasgn, :ivasgn, :cvasgn, :gvasgn, :casgn, :masgn].freeze
- SHORTHAND_ASGN_NODES =
[:op_asgn, :or_asgn, :and_asgn].freeze
- ASGN_NODES =
(EQUALS_ASGN_NODES + SHORTHAND_ASGN_NODES).freeze
- OPERATOR_METHODS =
phrogz.net/programmingruby/language.html#table_18.4 Backtick is added last just to help editors parse this code.
%w( | ^ & <=> == === =~ > >= < <= << >> + - * / % ** ~ +@ -@ [] []= ! != !~ ).map(&:to_sym).push(:'`').freeze
- LITERAL_REGEX =
Match literal regex characters, not including anchors, character classes, alternatives, groups, repetitions, references, etc
/[\w\s\-,"'!#%&<>=;:`~]|\\[^AbBdDgGhHkpPRwWXsSzZS0-9]/
Class Method Summary collapse
- .begins_its_line?(range) ⇒ Boolean
- .block_length(block_node) ⇒ Object
- .comment_line?(line_source) ⇒ Boolean
- .compatible_external_encoding_for?(src) ⇒ Boolean
- .directions(side) ⇒ Object
-
.double_quotes_acceptable?(string) ⇒ Boolean
If double quoted string literals are found in Ruby code, and they are not the preferred style, should they be flagged?.
-
.double_quotes_required?(string) ⇒ Boolean
If converting a string to Ruby string literal source code, must double quotes be used?.
-
.effective_column(range) ⇒ Object
Returns the column attribute of the range, except if the range is on the first line and there’s a byte order mark at the beginning of that line, in which case 1 is subtracted from the column value.
- .ends_its_line?(range) ⇒ Boolean
- .escape_string(string) ⇒ Object
-
.first_part_of_call_chain(node) ⇒ Object
Returns, for example, a bare
ifnode if the given node is anifwith calls chained to the end of it. - .hard_to_type?(string) ⇒ Boolean
- .interpret_string_escapes(string) ⇒ Object
- .line_distance(n1, n2) ⇒ Object
- .line_range(arg) ⇒ Object
- .move_pos(src, pos, step, condition, regexp) ⇒ Object
- .needs_escaping?(string) ⇒ Boolean
-
.numeric_range_size(range) ⇒ Object
Range#size is not available prior to Ruby 2.0.
- .on_node(syms, sexp, excludes = []) {|sexp| ... } ⇒ Object
- .operator?(symbol) ⇒ Boolean
- .parentheses?(node) ⇒ Boolean
- .parenthesized_call?(send) ⇒ Boolean
- .preceed?(n1, n2) ⇒ Boolean
- .range_between(start_pos, end_pos) ⇒ Object
- .range_with_surrounding_comma(range, side = :both) ⇒ Object
- .range_with_surrounding_space(range, side = :both, with_newline = true) ⇒ Object
- .same_line?(n1, n2) ⇒ Boolean
- .source_range(source_buffer, line_number, column, length = 1) ⇒ Object
- .strip_quotes(str) ⇒ Object
- .stripped_source_upto(line) ⇒ Object
- .to_string_literal(string) ⇒ Object
- .to_symbol_literal(string) ⇒ Object
- .within_node?(inner, outer) ⇒ Boolean
Methods included from Sexp
Methods included from PathUtil
absolute?, match_path?, relative_path
Class Method Details
.begins_its_line?(range) ⇒ Boolean
165 166 167 |
# File 'lib/rubocop/cop/util.rb', line 165 def begins_its_line?(range) (range.source_line =~ /\S/) == range.column end |
.block_length(block_node) ⇒ Object
47 48 49 |
# File 'lib/rubocop/cop/util.rb', line 47 def block_length(block_node) block_node.loc.end.line - block_node.loc.begin.line end |
.comment_line?(line_source) ⇒ Boolean
51 52 53 |
# File 'lib/rubocop/cop/util.rb', line 51 def comment_line?(line_source) line_source =~ /^\s*#/ end |
.compatible_external_encoding_for?(src) ⇒ Boolean
270 271 272 273 |
# File 'lib/rubocop/cop/util.rb', line 270 def compatible_external_encoding_for?(src) src = src.dup if RUBY_VERSION < '2.3' src.force_encoding(Encoding.default_external).valid_encoding? end |
.directions(side) ⇒ Object
157 158 159 160 161 162 163 |
# File 'lib/rubocop/cop/util.rb', line 157 def directions(side) if side == :both [true, true] else [side == :left, side == :right] end end |
.double_quotes_acceptable?(string) ⇒ Boolean
If double quoted string literals are found in Ruby code, and they are not the preferred style, should they be flagged?
220 221 222 |
# File 'lib/rubocop/cop/util.rb', line 220 def double_quotes_acceptable?(string) needs_escaping?(string) || hard_to_type?(string) end |
.double_quotes_required?(string) ⇒ Boolean
If converting a string to Ruby string literal source code, must double quotes be used?
208 209 210 211 212 213 214 215 216 |
# File 'lib/rubocop/cop/util.rb', line 208 def double_quotes_required?(string) # Double quotes are required for strings which either: # - Contain single quotes # - Contain non-printable characters, which must use an escape # Regex matches IF there is a ' or there is a \\ in the string that is # not preceded/followed by another \\ (e.g. "\\x34") but not "\\\\". string =~ /'|(?<! \\) \\{2}* \\ (?![\\"])/x end |
.effective_column(range) ⇒ Object
Returns the column attribute of the range, except if the range is on the first line and there’s a byte order mark at the beginning of that line, in which case 1 is subtracted from the column value. This gives the column as it appears when viewing the file in an editor.
109 110 111 112 113 114 115 116 |
# File 'lib/rubocop/cop/util.rb', line 109 def effective_column(range) if range.line == 1 && @processed_source.raw_source.codepoints.first == BYTE_ORDER_MARK range.column - 1 else range.column end end |
.ends_its_line?(range) ⇒ Boolean
169 170 171 172 |
# File 'lib/rubocop/cop/util.rb', line 169 def ends_its_line?(range) line = range.source_buffer.source_line(range.last_line) (line =~ /\s*\z/) == range.last_column end |
.escape_string(string) ⇒ Object
232 233 234 |
# File 'lib/rubocop/cop/util.rb', line 232 def escape_string(string) string.inspect[1..-2].tap { |s| s.gsub!(/\\"/, '"') } end |
.first_part_of_call_chain(node) ⇒ Object
Returns, for example, a bare if node if the given node is an if with calls chained to the end of it.
182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 |
# File 'lib/rubocop/cop/util.rb', line 182 def first_part_of_call_chain(node) while node case node.type when :send receiver, _method_name, _args = *node node = receiver when :block method, _args, _body = *node node = method else break end end node end |
.hard_to_type?(string) ⇒ Boolean
224 225 226 |
# File 'lib/rubocop/cop/util.rb', line 224 def hard_to_type?(string) string.codepoints.any? { |cp| cp < 32 || cp > 126 } end |
.interpret_string_escapes(string) ⇒ Object
248 249 250 |
# File 'lib/rubocop/cop/util.rb', line 248 def interpret_string_escapes(string) StringInterpreter.interpret(string) end |
.line_distance(n1, n2) ⇒ Object
258 259 260 |
# File 'lib/rubocop/cop/util.rb', line 258 def line_distance(n1, n2) n2.loc.line - n1.loc.line end |
.line_range(arg) ⇒ Object
55 56 57 58 59 60 61 62 63 64 65 66 |
# File 'lib/rubocop/cop/util.rb', line 55 def line_range(arg) source_range = case arg when Parser::Source::Range arg when Parser::AST::Node arg.source_range else raise ArgumentError, "Invalid argument #{arg}" end source_range.begin.line..source_range.end.line end |
.move_pos(src, pos, step, condition, regexp) ⇒ Object
151 152 153 154 155 |
# File 'lib/rubocop/cop/util.rb', line 151 def move_pos(src, pos, step, condition, regexp) offset = step == -1 ? -1 : 0 pos += step while condition && src[pos + offset] =~ regexp pos < 0 ? 0 : pos end |
.needs_escaping?(string) ⇒ Boolean
228 229 230 |
# File 'lib/rubocop/cop/util.rb', line 228 def needs_escaping?(string) double_quotes_required?(escape_string(string)) end |
.numeric_range_size(range) ⇒ Object
Range#size is not available prior to Ruby 2.0.
199 200 201 202 203 204 |
# File 'lib/rubocop/cop/util.rb', line 199 def numeric_range_size(range) size = range.end - range.begin size += 1 unless range.exclude_end? size = 0 if size < 0 size end |
.on_node(syms, sexp, excludes = []) {|sexp| ... } ⇒ Object
77 78 79 80 81 82 83 84 |
# File 'lib/rubocop/cop/util.rb', line 77 def on_node(syms, sexp, excludes = [], &block) return to_enum(:on_node, syms, sexp, excludes) unless block_given? yield sexp if Array(syms).include?(sexp.type) return if Array(excludes).include?(sexp.type) sexp.each_child_node { |elem| on_node(syms, elem, excludes, &block) } end |
.operator?(symbol) ⇒ Boolean
31 32 33 |
# File 'lib/rubocop/cop/util.rb', line 31 def operator?(symbol) OPERATOR_METHODS.include?(symbol) end |
.parentheses?(node) ⇒ Boolean
68 69 70 71 |
# File 'lib/rubocop/cop/util.rb', line 68 def parentheses?(node) node.loc.respond_to?(:end) && node.loc.end && node.loc.end.is?(')'.freeze) end |
.parenthesized_call?(send) ⇒ Boolean
73 74 75 |
# File 'lib/rubocop/cop/util.rb', line 73 def parenthesized_call?(send) send.loc.begin && send.loc.begin.is?('(') end |
.preceed?(n1, n2) ⇒ Boolean
262 263 264 |
# File 'lib/rubocop/cop/util.rb', line 262 def preceed?(n1, n2) line_distance(n1, n2) == 1 end |
.range_between(start_pos, end_pos) ⇒ Object
118 119 120 |
# File 'lib/rubocop/cop/util.rb', line 118 def range_between(start_pos, end_pos) Parser::Source::Range.new(processed_source.buffer, start_pos, end_pos) end |
.range_with_surrounding_comma(range, side = :both) ⇒ Object
122 123 124 125 126 127 128 129 130 131 132 133 134 |
# File 'lib/rubocop/cop/util.rb', line 122 def range_with_surrounding_comma(range, side = :both) buffer = @processed_source.buffer src = buffer.source go_left, go_right = directions(side) begin_pos = range.begin_pos end_pos = range.end_pos begin_pos = move_pos(src, begin_pos, -1, go_left, /,/) end_pos = move_pos(src, end_pos, 1, go_right, /,/) Parser::Source::Range.new(buffer, begin_pos, end_pos) end |
.range_with_surrounding_space(range, side = :both, with_newline = true) ⇒ Object
136 137 138 139 140 141 142 143 144 145 146 147 148 149 |
# File 'lib/rubocop/cop/util.rb', line 136 def range_with_surrounding_space(range, side = :both, with_newline = true) buffer = @processed_source.buffer src = buffer.source go_left, go_right = directions(side) begin_pos = range.begin_pos end_pos = range.end_pos begin_pos = move_pos(src, begin_pos, -1, go_left, /[ \t]/) begin_pos = move_pos(src, begin_pos, -1, go_left && with_newline, /\n/) end_pos = move_pos(src, end_pos, 1, go_right, /[ \t]/) end_pos = move_pos(src, end_pos, 1, go_right && with_newline, /\n/) Parser::Source::Range.new(buffer, begin_pos, end_pos) end |
.same_line?(n1, n2) ⇒ Boolean
252 253 254 255 256 |
# File 'lib/rubocop/cop/util.rb', line 252 def same_line?(n1, n2) n1.respond_to?(:loc) && n2.respond_to?(:loc) && n1.loc.line == n2.loc.line end |
.source_range(source_buffer, line_number, column, length = 1) ⇒ Object
86 87 88 89 90 91 92 93 94 95 96 97 98 99 100 101 102 103 |
# File 'lib/rubocop/cop/util.rb', line 86 def source_range(source_buffer, line_number, column, length = 1) if column.is_a?(Range) column_index = column.begin length = numeric_range_size(column) else column_index = column end line_begin_pos = if line_number.zero? 0 else source_buffer.line_range(line_number).begin_pos end begin_pos = line_begin_pos + column_index end_pos = begin_pos + length Parser::Source::Range.new(source_buffer, begin_pos, end_pos) end |
.strip_quotes(str) ⇒ Object
35 36 37 38 39 40 41 42 43 44 45 |
# File 'lib/rubocop/cop/util.rb', line 35 def strip_quotes(str) if str[0] == '"' || str[0] == "'" str[0] = '' else # we're dealing with %q or %Q str[0, 3] = '' end str[-1] = '' str end |
.stripped_source_upto(line) ⇒ Object
266 267 268 |
# File 'lib/rubocop/cop/util.rb', line 266 def stripped_source_upto(line) processed_source[0..line].map(&:strip) end |
.to_string_literal(string) ⇒ Object
236 237 238 239 240 241 242 |
# File 'lib/rubocop/cop/util.rb', line 236 def to_string_literal(string) if needs_escaping?(string) && compatible_external_encoding_for?(string) string.inspect else "'#{string.gsub('\\') { '\\\\' }}'" end end |
.to_symbol_literal(string) ⇒ Object
244 245 246 |
# File 'lib/rubocop/cop/util.rb', line 244 def to_symbol_literal(string) ":#{string.to_sym}" end |
.within_node?(inner, outer) ⇒ Boolean
174 175 176 177 178 |
# File 'lib/rubocop/cop/util.rb', line 174 def within_node?(inner, outer) o = outer.is_a?(Node) ? outer.source_range : outer i = inner.is_a?(Node) ? inner.source_range : inner i.begin_pos >= o.begin_pos && i.end_pos <= o.end_pos end |