Class: RuboCop::Formatter::EmacsStyleFormatter Private
- Inherits:
-
BaseFormatter
- Object
- BaseFormatter
- RuboCop::Formatter::EmacsStyleFormatter
- Defined in:
- lib/rubocop/formatter/emacs_style_formatter.rb
Overview
This class is part of a private API. You should avoid using this class if possible, as it may be removed or be changed in the future.
This formatter displays the report data in format that’s easy to process in the Emacs text editor. The output is machine-parsable.
Instance Attribute Summary
Attributes inherited from BaseFormatter
Instance Method Summary collapse
Methods inherited from BaseFormatter
#file_started, #finished, #initialize, #started
Constructor Details
This class inherits a constructor from RuboCop::Formatter::BaseFormatter
Instance Method Details
#file_finished(file, offenses) ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
9 10 11 12 13 14 15 16 17 18 19 20 |
# File 'lib/rubocop/formatter/emacs_style_formatter.rb', line 9 def file_finished(file, offenses) offenses.each do |o| output.printf( "%<path>s:%<line>d:%<column>d: %<severity>s: %<message>s\n", path: file, line: o.line, column: o.real_column, severity: o.severity.code, message: (o) ) end end |