Class: RuboCop::Formatter::OffenseCountFormatter Private
- Inherits:
-
BaseFormatter
- Object
- BaseFormatter
- RuboCop::Formatter::OffenseCountFormatter
- Defined in:
- lib/rubocop/formatter/offense_count_formatter.rb
Overview
This class is part of a private API. You should avoid using this class if possible, as it may be removed or be changed in the future.
This formatter displays the list of offended cops with a count of how many offenses of their kind were found. Ordered by desc offense count
Here’s the format:
26 LineLength 3 OneLineConditional – 29 Total in 5 files
Instance Attribute Summary collapse
- #offense_counts ⇒ Object readonly private
Attributes inherited from BaseFormatter
Instance Method Summary collapse
- #file_finished(_file, offenses) ⇒ Object private
- #finished(_inspected_files) ⇒ Object private
-
#ordered_offense_counts(offense_counts) ⇒ Object
private
rubocop:enable Metrics/AbcSize.
-
#report_summary(offense_counts, offending_files_count) ⇒ Object
private
rubocop:disable Metrics/AbcSize.
- #started(target_files) ⇒ Object private
- #total_offense_count(offense_counts) ⇒ Object private
Methods inherited from BaseFormatter
Constructor Details
This class inherits a constructor from RuboCop::Formatter::BaseFormatter
Instance Attribute Details
#offense_counts ⇒ Object (readonly)
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
17 18 19 |
# File 'lib/rubocop/formatter/offense_count_formatter.rb', line 17 def offense_counts @offense_counts end |
Instance Method Details
#file_finished(_file, offenses) ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
42 43 44 45 46 47 48 49 |
# File 'lib/rubocop/formatter/offense_count_formatter.rb', line 42 def file_finished(_file, offenses) offenses.each { |o| @offense_counts[o.cop_name] += 1 } if [:display_style_guide] offenses.each { |o| @style_guide_links[o.cop_name] ||= o.[/ \(http\S+\)\Z/] } end @offending_files_count += 1 unless offenses.empty? @progressbar.increment if instance_variable_defined?(:@progressbar) end |
#finished(_inspected_files) ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
51 52 53 |
# File 'lib/rubocop/formatter/offense_count_formatter.rb', line 51 def finished(_inspected_files) report_summary(@offense_counts, @offending_files_count) end |
#ordered_offense_counts(offense_counts) ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
rubocop:enable Metrics/AbcSize
74 75 76 |
# File 'lib/rubocop/formatter/offense_count_formatter.rb', line 74 def ordered_offense_counts(offense_counts) offense_counts.sort_by { |k, v| [-v, k] }.to_h end |
#report_summary(offense_counts, offending_files_count) ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
rubocop:disable Metrics/AbcSize
56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 |
# File 'lib/rubocop/formatter/offense_count_formatter.rb', line 56 def report_summary(offense_counts, offending_files_count) per_cop_counts = ordered_offense_counts(offense_counts) total_count = total_offense_count(offense_counts) output.puts column_width = total_count.to_s.length + 2 per_cop_counts.each do |cop_name, count| output.puts "#{count.to_s.ljust(column_width)}#{cop_name}" \ "#{@style_guide_links[cop_name]}\n" end output.puts '--' output.puts "#{total_count} Total in #{offending_files_count} files" output.puts end |
#started(target_files) ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 |
# File 'lib/rubocop/formatter/offense_count_formatter.rb', line 19 def started(target_files) super @offense_counts = Hash.new(0) @offending_files_count = 0 @style_guide_links = {} return unless output.tty? file_phrase = target_files.count == 1 ? 'file' : 'files' # 185/407 files |====== 45 ======> | ETA: 00:00:04 # %c / %C | %w > %i | %e = " %c/%C #{file_phrase} |%w>%i| %e " @progressbar = ProgressBar.create( output: output, total: target_files.count, format: , autostart: false ) @progressbar.start end |
#total_offense_count(offense_counts) ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
78 79 80 |
# File 'lib/rubocop/formatter/offense_count_formatter.rb', line 78 def total_offense_count(offense_counts) offense_counts.values.sum end |