Class: Rubocop::Cop::Style::DefWithParentheses
- Includes:
- CheckMethods
- Defined in:
- lib/rubocop/cop/style/def_parentheses.rb
Overview
This cop checks for parentheses in the definition of a method, that does not take any arguments. Both instance and class/singleton methods are checked.
Constant Summary collapse
- MSG =
"Omit the parentheses in defs when the method doesn't accept " + 'any arguments.'
Constants inherited from Cop
Constants included from Util
Instance Attribute Summary
Attributes inherited from Cop
#config, #corrections, #offences, #processed_source
Instance Method Summary collapse
Methods included from CheckMethods
Methods inherited from Cop
#add_offence, all, #autocorrect?, #config_to_allow_offences, #config_to_allow_offences=, #cop_config, cop_name, #cop_name, cop_type, #debug?, #exclude_file?, #exclude_paths, #ignore_node, #include_file?, #include_paths, inherited, #initialize, lint?, #message, non_rails, rails?, #relevant_file?, style?, #support_autocorrect?
Methods included from Util
block_length, command?, comment_line?, const_name, first_part_of_call_chain, lambda?, lambda_or_proc?, line_range, on_node, proc?, range_with_surrounding_space, source_range, strip_quotes
Constructor Details
This class inherits a constructor from Rubocop::Cop::Cop
Instance Method Details
#autocorrect(node) ⇒ Object
24 25 26 27 28 29 |
# File 'lib/rubocop/cop/style/def_parentheses.rb', line 24 def autocorrect(node) @corrections << lambda do |corrector| corrector.remove(node.loc.begin) corrector.remove(node.loc.end) end end |
#check(node, _method_name, args, _body) ⇒ Object
15 16 17 18 19 20 21 22 |
# File 'lib/rubocop/cop/style/def_parentheses.rb', line 15 def check(node, _method_name, args, _body) start_line = node.loc.keyword.line end_line = node.loc.end.line return if start_line == end_line add_offence(args, :begin) if args.children == [] && args.loc.begin end |