Class: Rubocop::Cop::Style::IfUnlessModifier
- Includes:
- FavorModifier
- Defined in:
- lib/rubocop/cop/style/favor_modifier.rb
Overview
Checks for if and unless statements that would fit on one line if written as a modifier if/unless.
Constant Summary
Constants inherited from Cop
Constants included from Util
Instance Attribute Summary
Attributes inherited from Cop
#config, #corrections, #offences, #processed_source
Instance Method Summary collapse
Methods included from FavorModifier
#body_has_comment?, #body_length, #check, #conditional_length, #length, #max_line_length
Methods included from IfNode
#elsif?, #if_else?, #modifier_if?, #ternary_op?
Methods inherited from Cop
#add_offence, all, #autocorrect?, #config_to_allow_offences, #config_to_allow_offences=, #cop_config, cop_name, #cop_name, cop_type, #debug?, #exclude_file?, #exclude_paths, #ignore_node, #include_file?, #include_paths, inherited, #initialize, lint?, #message, non_rails, rails?, #relevant_file?, style?, #support_autocorrect?
Methods included from Util
block_length, command?, comment_line?, const_name, first_part_of_call_chain, lambda?, lambda_or_proc?, line_range, on_node, proc?, range_with_surrounding_space, source_range, strip_quotes
Constructor Details
This class inherits a constructor from Rubocop::Cop::Cop
Instance Method Details
#error_message ⇒ Object
75 76 77 78 |
# File 'lib/rubocop/cop/style/favor_modifier.rb', line 75 def 'Favor modifier if/unless usage when you have a single-line body. ' + 'Another good alternative is the usage of control flow &&/||.' end |
#investigate(processed_source) ⇒ Object
80 81 82 83 84 85 86 87 88 89 90 91 92 93 |
# File 'lib/rubocop/cop/style/favor_modifier.rb', line 80 def investigate(processed_source) return unless processed_source.ast on_node(:if, processed_source.ast) do |node| # discard ternary ops, if/else and modifier if/unless nodes next if ternary_op?(node) next if modifier_if?(node) next if elsif?(node) next if if_else?(node) if check(node, processed_source.comments) add_offence(node, :keyword, ) end end end |