Class: Rubocop::Cop::Style::WhileUntilModifier
- Includes:
- FavorModifier
- Defined in:
- lib/rubocop/cop/style/favor_modifier.rb
Overview
Checks for while and until statements that would fit on one line if written as a modifier while/until.
Constant Summary collapse
- MSG =
'Favor modifier while/until usage when you have a single-line body.'
Constants inherited from Cop
Constants included from Util
Instance Attribute Summary
Attributes inherited from Cop
#config, #corrections, #offences, #processed_source
Instance Method Summary collapse
Methods included from FavorModifier
#body_has_comment?, #body_length, #check, #conditional_length, #length, #max_line_length
Methods included from IfNode
#elsif?, #if_else?, #modifier_if?, #ternary_op?
Methods inherited from Cop
#add_offence, all, #autocorrect?, #config_to_allow_offences, #config_to_allow_offences=, #cop_config, cop_name, #cop_name, cop_type, #debug?, #exclude_file?, #exclude_paths, #ignore_node, #include_file?, #include_paths, inherited, #initialize, lint?, #message, non_rails, rails?, #relevant_file?, style?, #support_autocorrect?
Methods included from Util
block_length, command?, comment_line?, const_name, first_part_of_call_chain, lambda?, lambda_or_proc?, line_range, on_node, proc?, range_with_surrounding_space, source_range, strip_quotes
Constructor Details
This class inherits a constructor from Rubocop::Cop::Cop
Instance Method Details
#investigate(processed_source) ⇒ Object
104 105 106 107 108 109 110 111 112 113 114 |
# File 'lib/rubocop/cop/style/favor_modifier.rb', line 104 def investigate(processed_source) return unless processed_source.ast on_node([:while, :until], processed_source.ast) do |node| # discard modifier while/until next unless node.loc.end if check(node, processed_source.comments) add_offence(node, :keyword) end end end |